@@ -13,11 +13,15 @@
 #include <stdlib.h>
 #endif
 #if TIME_WITH_SYS_TIME
-# include <sys/time.h>
+# ifdef WIN32
+#  include <sys/timeb.h>
+# else
+#  include <sys/time.h>
+# endif
 # include <time.h>
 #else
 # if HAVE_SYS_TIME_H
-#  include <sys/time.h>
+#  include <sys/time.h> 
 # else
 #  include <time.h>
 # endif
@@ -25,6 +29,15 @@
 #if HAVE_NETINET_IN_H
 #include <netinet/in.h>
 #endif
+#if HAVE_WINSOCK_H
+#include <winsock.h>
+#endif
+#if HAVE_SYS_SOCKET_H
+#include <sys/socket.h>
+#endif
+#if HAVE_DMALLOC_H
+#include <dmalloc.h>
+#endif
 
 #include "asn1.h"
 #include "snmp_api.h"
@@ -52,7 +65,6 @@ struct snmp_session *
 find_agentx_session( struct snmp_session *session, int sessid)
 {
     struct snmp_session *sp;
-    
     for ( sp = session->subsession ; sp != NULL ; sp = sp->next ) {
         if ( sp->sessid == sessid )
 	    return sp;
@@ -67,6 +79,7 @@ open_agentx_session(struct snmp_session *session, struct snmp_pdu *pdu)
     struct snmp_session *sp;
     struct timeval now;
 
+    DEBUGMSGTL(("agentx:open_agentx_session","open %p\n", session));
     sp = (struct snmp_session *)malloc( sizeof( struct snmp_session ));
     if ( sp == NULL ) {
         session->s_snmp_errno = AGENTX_ERR_OPEN_FAILED;
@@ -89,7 +102,7 @@ open_agentx_session(struct snmp_session *session, struct snmp_pdu *pdu)
     sp->securityAuthProto =
 	snmp_duplicate_objid(pdu->variables->name, pdu->variables->name_length);
     sp->securityAuthProtoLen = pdu->variables->name_length;
-    sp->securityName = strdup( pdu->variables->val.string );
+    sp->securityName = strdup((char *) pdu->variables->val.string );
     gettimeofday(&now, NULL);
     sp->engineTime = calculate_time_diff( &now, &starttime );
 
@@ -97,6 +110,7 @@ open_agentx_session(struct snmp_session *session, struct snmp_pdu *pdu)
     sp->flags     |= SNMP_FLAGS_SUBSESSION;
     sp->next       = session->subsession;
     session->subsession = sp;
+    DEBUGMSGTL(("agentx:open_agentx_session","opened %p = %d\n", sp, sp->sessid));
 
     return sp->sessid;
 }
@@ -106,6 +120,7 @@ close_agentx_session(struct snmp_session *session, int sessid)
 {
     struct snmp_session *sp, *prev = NULL;
     
+    DEBUGMSGTL(("agentx:close_agentx_session","close %p, %d\n", session, sessid));
     if ( sessid == -1 ) {
 	unregister_mibs_by_session( session );
 	unregister_index_by_session( session );
@@ -113,7 +128,7 @@ close_agentx_session(struct snmp_session *session, int sessid)
 
 	return AGENTX_ERR_NOERROR;
     }
-    else
+
     for ( sp = session->subsession ; sp != NULL ; prev = sp, sp = sp->next ) {
         if ( sp->sessid == sessid ) {
 
@@ -124,6 +139,10 @@ close_agentx_session(struct snmp_session *session, int sessid)
 	        prev->next = sp->next;
 	    else
 	    	session->subsession = sp->next;
+	    if (sp->securityAuthProto)
+		free(sp->securityAuthProto);
+	    if (sp->securityName)
+		free(sp->securityName);
 	    free( sp );
 	    
 	    return AGENTX_ERR_NOERROR;
@@ -157,7 +176,7 @@ register_agentx_list(struct snmp_session *session, struct snmp_pdu *pdu)
 			 sizeof(agentx_varlist[0]), 1,
 			 pdu->variables->name, pdu->variables->name_length,
 			 pdu->priority, pdu->range_subid, ubound, sp,
-			 pdu->community, pdu->time,
+			 (char *)pdu->community, pdu->time,
 			 pdu->flags&AGENTX_MSG_FLAG_INSTANCE_REGISTER)) {
 
 	case MIB_REGISTERED_OK:
@@ -189,7 +208,7 @@ unregister_agentx_list(struct snmp_session *session, struct snmp_pdu *pdu)
     switch (unregister_mib_context(pdu->variables->name,
     		       pdu->variables->name_length,
 		       pdu->priority, pdu->range_subid, ubound,
-		       pdu->community)) {
+		       (char *)pdu->community)) {
 	case MIB_UNREGISTERED_OK:
 				return AGENTX_ERR_NOERROR;
 	case MIB_NO_SUCH_REGISTRATION:
