@@ -9,8 +9,18 @@
 #else
 #include <strings.h>
 #endif
+
+#ifdef dynix
+#if HAVE_SYS_SELECT_H
+#include <sys/select.h>
+#endif
+#endif
+
 #include "host_res.h"
 #include "hr_print.h"
+#include "struct.h"
+#include "tools.h"
+#include "util_funcs.h"
 
 #define HRPRINT_MONOTONICALLY_INCREASING
 
@@ -29,7 +39,9 @@ int printer_status(int);
 int printer_detail_status(int);
 int printer_errors(int);
 int header_hrprint (struct variable *,oid *, size_t *, int, size_t *, WriteMethod **);
-
+#ifdef HAVE_LPSTAT
+FILE * run_lpstat(void);
+#endif
 
 	/*********************
 	 *
@@ -205,14 +217,14 @@ Init_HR_Print (void)
     }
   
 #if HAVE_LPSTAT
-    if ((p = popen(LPSTAT_PATH " -v", "r")) != NULL) {
+    if ((p = run_lpstat()) != NULL) {
 	char buf[BUFSIZ], ptr[BUFSIZ];
 	while (fgets(buf, sizeof buf, p)) {
 	    sscanf(buf, "%*s %*s %[^:]", ptr);
 #elif HAVE_CGETNEXT
     {
 	char *buf = NULL, *ptr;
-	while (cgetnext(&buf, caps)) {
+	while (cgetnext(&buf, caps) > 0) {
 	    if ((ptr = strchr(buf, ':'))) *ptr = 0;
 	    if ((ptr = strchr(buf, '|'))) *ptr = 0;
 	    ptr = buf;
@@ -301,3 +313,25 @@ int printer_errors(int idx)
 {
     return 0;
 }
+
+#ifdef        HAVE_LPSTAT
+/*
+ * Run the lpstat command. If compiled with EXCACHE support, this
+ * will actually cache the output for a while which helps a lot
+ * with snmpbulkwalk (in fact, it keeps the client from exiting
+ * due to timeouts).
+ */
+FILE *
+run_lpstat(void)
+{
+    struct extensible	ex;
+    int			fd;
+
+    memset(&ex, 0, sizeof(ex));
+    strcpy(ex.command, LPSTAT_PATH " -v");
+    if ((fd = get_exec_output(&ex)) < 0)
+    	return NULL;
+
+    return fdopen(fd, "r");
+}
+#endif
