@@ -50,6 +50,8 @@
 #ifdef cygwin
 #include <windows.h>
 #include <sys/cygwin.h>
+#include <tlhelp32.h>
+#include <psapi.h>
 #endif
 
 #if _SLASH_PROC_METHOD_
@@ -64,6 +66,7 @@
 
 #include <stdio.h>
 
+#include "snmp_logging.h"
 #include "host_res.h"
 #include "hr_swrun.h"
 #include "auto_nlist.h"
@@ -83,13 +86,17 @@ void  End_HR_SWRun (void);
 int header_hrswrun (struct variable *,oid *, size_t *, int, size_t *, WriteMethod **);
 int header_hrswrunEntry (struct variable *,oid *, size_t *, int, size_t *, WriteMethod **);
 
+#ifdef dynix
+pid_t nextproc;
+static prpsinfo_t lowpsinfo, mypsinfo;
+#endif
 #ifdef cygwin
 static struct external_pinfo *curproc;
 static struct external_pinfo lowproc;
 #elif !defined(linux)
 static int LowProcIndex;
 #endif
-#ifdef hpux10
+#if defined(hpux10) || defined(hpux11)
 struct pst_status *proc_table;
 struct pst_dynamic pst_dyn;
 #elif HAVE_KVM_GETPROCS
@@ -99,7 +106,9 @@ int *proc_table;
 #else
 struct proc *proc_table;
 #endif
+#if !defined(dynix)
 int current_proc_entry;
+#endif
 
 
 #define	HRSWRUN_OSINDEX		1
@@ -115,6 +124,8 @@ int current_proc_entry;
 #define	HRSWRUNPERF_CPU		9
 #define	HRSWRUNPERF_MEM		10
 
+#define HRSWRUN_PARAMS_MAX	128
+
 struct variable4 hrswrun_variables[] = {
     { HRSWRUN_OSINDEX,   ASN_INTEGER, RONLY, var_hrswrun, 1, {1}},
     { HRSWRUN_INDEX,     ASN_INTEGER, RONLY, var_hrswrun, 3, {2,1,1}},
@@ -134,17 +145,158 @@ struct variable4 hrswrunperf_variables[] = {
 oid hrswrun_variables_oid[]     = { 1,3,6,1,2,1,25,4 };
 oid hrswrunperf_variables_oid[] = { 1,3,6,1,2,1,25,5 };
 
+#ifdef cygwin
+
+/*
+ * a lot of this is "stolen" from cygwin ps.cc
+ */
+
+typedef BOOL (WINAPI *ENUMPROCESSMODULES) (
+    HANDLE hProcess,
+    HMODULE *lphModule,
+    DWORD cb,
+    LPDWORD lpcbNeeded);
+
+typedef DWORD (WINAPI *GETMODULEFILENAME) (
+    HANDLE hProcess,
+    HMODULE hModule,
+    LPTSTR lpstrFIleName,
+    DWORD nSize);
+
+typedef DWORD (WINAPI *GETPROCESSMEMORYINFO) (
+    HANDLE hProcess,
+    PPROCESS_MEMORY_COUNTERS pmc,
+    DWORD nSize);
+
+typedef HANDLE (WINAPI *CREATESNAPSHOT) (
+    DWORD dwFlags,
+    DWORD th32ProcessID);
+
+typedef BOOL (WINAPI *PROCESSWALK) (
+    HANDLE hSnapshot,
+    LPPROCESSENTRY32 lppe);
+
+ENUMPROCESSMODULES myEnumProcessModules;
+GETMODULEFILENAME myGetModuleFileNameEx;
+CREATESNAPSHOT myCreateToolhelp32Snapshot;
+PROCESSWALK myProcess32First;
+PROCESSWALK myProcess32Next;
+GETPROCESSMEMORYINFO myGetProcessMemoryInfo = NULL;
+cygwin_getinfo_types query = CW_GETPINFO;
+
+static BOOL WINAPI dummyprocessmodules(
+    HANDLE hProcess,
+    HMODULE *lphModule,
+    DWORD cb,
+    LPDWORD lpcbNeeded)
+{
+    lphModule[0] = (HMODULE) *lpcbNeeded;
+    *lpcbNeeded = 1;
+    return 1;
+}
+
+static DWORD WINAPI GetModuleFileNameEx95(
+    HANDLE hProcess,
+    HMODULE hModule,
+    LPTSTR lpstrFileName,
+    DWORD n)
+{
+    HANDLE h;
+    DWORD pid = (DWORD) hModule;
+    PROCESSENTRY32 proc;
+
+    h = myCreateToolhelp32Snapshot(TH32CS_SNAPPROCESS, 0);
+    if (!h) return 0;
+    proc.dwSize = sizeof(proc);
+    if (myProcess32First(h, &proc))
+      do
+        if (proc.th32ProcessID == pid) {
+	    CloseHandle(h);
+	    strcpy(lpstrFileName, proc.szExeFile);
+	    return 1;
+	}
+      while (myProcess32Next(h, &proc));
+    CloseHandle(h);
+    return 0;
+}
+
+#define FACTOR (0x19db1ded53ea710LL)
+#define NSPERSEC 10000000LL
+#define NSPERMSEC 10000LL
+
+static time_t __stdcall
+to_time_t(PFILETIME ptr)
+{
+    long rem;
+    long long x = ((long long)ptr->dwHighDateTime << 32) + ((unsigned)ptr->dwLowDateTime);
+    x -= FACTOR;
+    rem = x % NSPERSEC;
+    rem += NSPERSEC/2;
+    x /= NSPERSEC;
+    x += rem/NSPERSEC;
+    return x;
+}
+
+static long to_msec(PFILETIME ptr)
+{
+    long long x = ((long long)ptr->dwHighDateTime << 32) + (unsigned)ptr->dwLowDateTime;
+    x /= NSPERMSEC;
+    return x;
+}
+
+#endif /* cygwin */
+
 
 void init_hr_swrun(void)
 {
+#ifdef cygwin
+    OSVERSIONINFO ver;
+    HMODULE h;
+
+    memset(&ver, 0, sizeof ver);
+    ver.dwOSVersionInfoSize = sizeof ver;
+    GetVersionEx(&ver);
+
+    if (ver.dwPlatformId == VER_PLATFORM_WIN32_NT) {
+	h = LoadLibrary("psapi.dll");
+	if (h) {
+	    myEnumProcessModules = (ENUMPROCESSMODULES) GetProcAddress(h, "EnumProcessModules");
+	    myGetModuleFileNameEx = (GETMODULEFILENAME) GetProcAddress(h, "GetModuleFileNameExA");
+	    myGetProcessMemoryInfo = (GETPROCESSMEMORYINFO) GetProcAddress(h, "GetProcessMemoryInfo");
+	    if (myEnumProcessModules && myGetModuleFileNameEx)
+		query = CW_GETPINFO_FULL;
+	    else
+		snmp_log(LOG_ERR, "hr_swrun failed NT init\n");
+	}
+	else
+	    snmp_log(LOG_ERR, "hr_swrun failed to load psapi.dll\n");
+    }
+    else {
+	h = GetModuleHandle("KERNEL32.DLL");
+	myCreateToolhelp32Snapshot = (CREATESNAPSHOT) GetProcAddress(h, "CreateToolhelp32Snapshot");
+	myProcess32First = (PROCESSWALK) GetProcAddress(h, "Process32First");
+	myProcess32Next = (PROCESSWALK) GetProcAddress(h, "Process32Next");
+	myEnumProcessModules = dummyprocessmodules;
+	myGetModuleFileNameEx = GetModuleFileNameEx95;
+	if (myCreateToolhelp32Snapshot && myProcess32First && myProcess32Next)
+#if 0
+	    /* This doesn't work after all on Win98 SE */
+	    query = CW_GETPINFO_FULL;
+#else
+	    query = CW_GETPINFO;
+#endif
+	else
+	    snmp_log(LOG_ERR, "hr_swrun failed non-NT init\n");
+    }
+#endif /* cygwin */
 #ifdef PROC_SYMBOL
-  auto_nlist( PROC_SYMBOL,0,0 );
+    auto_nlist( PROC_SYMBOL,0,0 );
 #endif
 #ifdef NPROC_SYMBOL
-  auto_nlist( NPROC_SYMBOL,0,0 );
+    auto_nlist( NPROC_SYMBOL,0,0 );
 #endif
 
-  proc_table = 0;
+    proc_table = 0;
 
     REGISTER_MIB("host/hr_swrun", hrswrun_variables, variable4, hrswrun_variables_oid);
     REGISTER_MIB("host/hr_swrun", hrswrunperf_variables, variable4, hrswrunperf_variables_oid);
@@ -215,11 +367,18 @@ header_hrswrunEntry(struct variable *vp,
 		 */
     Init_HR_SWRun();
     for ( ;; ) {
+#ifdef dynix
+        DEBUGMSGTL(("host/hr_swrun::header_hrswrunEntry", "nextproc == %d\n", nextproc));
+#endif
         pid = Get_Next_HR_SWRun();
-#ifndef linux
+#if !defined(linux)
+#if !defined(dynix)
         DEBUGMSG(("host/hr_swrun",
                   "(index %d (entry #%d) ....", pid, current_proc_entry));
-#endif
+#else
+        DEBUGMSG(("host/hr_swrun", "pid %d; nextproc %d ....", pid, nextproc));
+#endif /* !dynix */
+#endif /* !linux */
         if ( pid == -1 )
 	    break;
 	newname[HRSWRUN_ENTRY_NAME_LENGTH] = pid;
@@ -230,6 +389,8 @@ header_hrswrunEntry(struct variable *vp,
 	    LowPid = pid;
 #ifdef cygwin
 	    lowproc = *curproc;
+#elif dynix
+            memcpy(&lowpsinfo, &mypsinfo, sizeof(struct prpsinfo));
 #elif !defined(linux)
 	    LowProcIndex = current_proc_entry-1;
 #endif
@@ -242,6 +403,8 @@ header_hrswrunEntry(struct variable *vp,
 	    LowPid = pid;
 #ifdef cygwin
 	    lowproc = *curproc;
+#elif dynix
+            memcpy(&lowpsinfo, &mypsinfo, sizeof(prpsinfo_t));
 #elif !defined(linux)
 	    LowProcIndex = current_proc_entry-1;
 #endif
@@ -250,6 +413,7 @@ header_hrswrunEntry(struct variable *vp,
 	}
 	DEBUGMSG(("host/hr_swrun", "\n"));
     }
+    End_HR_SWRun();
 
     if ( LowPid == -1 ) {
         DEBUGMSGTL(("host/hr_swrun", "... index out of range\n"));
@@ -366,6 +530,11 @@ var_hrswrun(struct variable *vp,
 	    cp = strchr( string, ' ');
 	    if ( cp != NULL )
 		*cp = '\0';
+#elif defined(dynix)
+	    sprintf(string, "%s", lowpsinfo.pr_fname);
+	    cp = strchr( string, ' ');
+	    if ( cp != NULL )
+		*cp = '\0';
 #elif defined(solaris2)
 #if _SLASH_PROC_METHOD_
 	    if (proc_buf)
@@ -397,10 +566,32 @@ var_hrswrun(struct variable *vp,
 		cp = strrchr(string, '/');
 		if (cp) strcpy(string, cp+1);
 	    }
-	    else {
-		/* get code from ps to get windows process path */
-		string[0] = 0;
+	    else if (query == CW_GETPINFO_FULL) {
+		DWORD n = lowproc.dwProcessId & 0xffff;
+		HANDLE h = OpenProcess(PROCESS_QUERY_INFORMATION | PROCESS_VM_READ,
+				       FALSE, n);
+
+		if (h) {
+		    HMODULE hm[1000];
+		    if (!myEnumProcessModules(h, hm, sizeof hm, &n)) {
+			snmp_log(LOG_DEBUG, "no module handle for %lu\n", n);
+			n = 0;
+		    }
+		    if (n && myGetModuleFileNameEx(h, hm[0], string, sizeof string)) {
+			cp = strrchr(string, '\\');
+			if (cp) strcpy(string, cp+1);
+		    }
+		    else
+			strcpy(string, "*** unknown");
+		    CloseHandle(h);
+		}
+		else {
+		    snmp_log(LOG_INFO, "no process handle for %lu\n", n);
+		    strcpy(string, "** unknown");
+		}
 	    }
+	    else
+		strcpy(string, "* unknown");
 	    cp = strchr(string, '\0') - 4;
 	    if (cp > string && strcasecmp(cp, ".exe") == 0)
 		*cp = '\0';
@@ -428,6 +619,12 @@ var_hrswrun(struct variable *vp,
 	    cp = strchr( string, ' ');
 	    if ( cp != NULL )
 		*cp = '\0';
+#elif defined(dynix)
+		/* Path not available - use argv[0] */
+	    sprintf(string, "%s", lowpsinfo.pr_psargs);
+	    cp = strchr( string, ' ');
+	    if ( cp != NULL )
+		*cp = '\0';
 #elif defined(solaris2)
 #ifdef _SLASH_PROC_METHOD_
 	    if (proc_buf)
@@ -469,10 +666,23 @@ var_hrswrun(struct variable *vp,
 		strcpy(string, "<defunct>");
 	    else if (lowproc.ppid)
 		cygwin_conv_to_posix_path(lowproc.progname, string);
-	    else {
-		/* get code from ps to get windows process path! */
-	    	string[0] = 0;
+	    else if (query == CW_GETPINFO_FULL) {
+		DWORD n = lowproc.dwProcessId & 0xFFFF;
+		HANDLE h = OpenProcess(PROCESS_QUERY_INFORMATION | PROCESS_VM_READ,
+				       FALSE, n);
+		if (h) {
+		    HMODULE hm[1000];
+		    if (!myEnumProcessModules(h, hm, sizeof hm, &n))
+			n = 0;
+		    if (!n || !myGetModuleFileNameEx(h, hm[0], string, sizeof string))
+			strcpy(string, "*** unknown");
+		    CloseHandle(h);
+		}
+		else
+		    strcpy(string, "** unknown");
 	    }
+	    else
+		strcpy(string, "* unknown");
 #else
 #if NO_DUMMY_VALUES
 	    return NULL;
@@ -486,7 +696,15 @@ var_hrswrun(struct variable *vp,
 	    cp = strchr( proc_buf.pst_cmd, ' ');
 	    if ( cp != NULL ) {
 		cp++;
-		sprintf(string, "%s", cp);
+		snprintf(string, HRSWRUN_PARAMS_MAX, "%s", cp);
+	    }
+	    else
+		string[0] = '\0';
+#elif defined(dynix)
+	    cp = strchr( lowpsinfo.pr_psargs, ' ');
+	    if ( cp != NULL ) {
+		cp++;
+		snprintf(string, HRSWRUN_PARAMS_MAX, "%s", cp);
 	    }
 	    else
 		string[0] = '\0';
@@ -494,7 +712,7 @@ var_hrswrun(struct variable *vp,
 #ifdef _SLASH_PROC_METHOD_
 	    if (proc_buf) {
 	        cp = strchr(proc_buf->pr_psargs, ' ');
-	        if (cp) strcpy(string, cp+1);
+	        if (cp) strncpy(string, cp+1, HRSWRUN_PARAMS_MAX);
 	        else string[0] = 0;
 	    } else
 		string[0] = 0;
@@ -502,15 +720,17 @@ var_hrswrun(struct variable *vp,
 	    cp = proc_buf->p_user.u_psargs;
 	    while (*cp && *cp != ' ') cp++;
 	    if (*cp == ' ') cp++;
-	    strcpy (string, cp);
+	    strncpy (string, cp, HRSWRUN_PARAMS_MAX);
 #endif
 #elif HAVE_KVM_GETPROCS
 	    string[0] = 0;
 	    argv = kvm_getargv(kd, proc_table+LowProcIndex, sizeof(string));
 	    if (argv) argv++;
 	    while (argv && *argv) {
+		int len = strlen(string);
 		if (string[0] != 0) strcat(string, " ");
-		strcat(string, *argv);
+		strncpy(string+len, *argv, HRSWRUN_PARAMS_MAX-len);
+		string[127] = 0;
 		argv++;
 	    }
 #elif defined(linux)
@@ -545,7 +765,7 @@ var_hrswrun(struct variable *vp,
 	    while ( *cp )
 		++cp;
 	    ++cp;
-	    strcpy( string, cp );
+	    strncpy( string, cp, HRSWRUN_PARAMS_MAX );
             fclose(fp);
 #elif defined(cygwin)
 	    string[0] = 0;
@@ -555,6 +775,7 @@ var_hrswrun(struct variable *vp,
 #endif
 	    sprintf(string, "-h -q -v");
 #endif
+	    string[HRSWRUN_PARAMS_MAX] = 0;
 	    *var_len = strlen(string);
 	    return (u_char *) string;
 	case HRSWRUN_TYPE:
@@ -574,7 +795,7 @@ var_hrswrun(struct variable *vp,
 	    else
 		long_return = 1; /* running */
 #elif !defined(linux)
-#ifdef hpux10
+#if defined(hpux10) || defined(hpux11)
 	    switch ( proc_table[LowProcIndex].pst_stat ) {
 		case PS_STOP:
 	    		long_return = 3;	/* notRunnable */
@@ -595,6 +816,8 @@ var_hrswrun(struct variable *vp,
 #else
 #if HAVE_KVM_GETPROCS
 	    switch ( proc_table[LowProcIndex].kp_proc.p_stat ) {
+#elif defined(dynix)
+	    switch ( lowpsinfo.pr_state ) {
 #elif defined(solaris2)
 #if _SLASH_PROC_METHOD_
 	    switch (proc_buf ? proc_buf->pr_lwp.pr_state : SIDL) {
@@ -608,13 +831,23 @@ var_hrswrun(struct variable *vp,
 	    		long_return = 3;	/* notRunnable */
 			break;
 		case 0:
+#ifdef SSWAP
+		case SSWAP:
+#endif
+#ifdef SSLEEP
 		case SSLEEP:
+#endif
 #ifdef SWAIT
 		case SWAIT:
 #endif
 	    		long_return = 2;	/* runnable */
 			break;
+#ifdef SACTIVE
+		case SACTIVE:
+#endif
+#ifdef SRUN
 		case SRUN:
+#endif
 #ifdef SONPROC
 		case SONPROC:
 #endif
@@ -666,6 +899,9 @@ var_hrswrun(struct variable *vp,
 				/*
 				 * Not convinced this is right, but....
 				 */
+#elif defined(dynix)
+	    long_return = lowpsinfo.pr_time.tv_sec * 100 +
+                          lowpsinfo.pr_time.tv_nsec/10000000;
 #elif defined(solaris2)
 #if _SLASH_PROC_METHOD_
 	    long_return = proc_buf ? proc_buf->pr_time.tv_sec * 100 +
@@ -699,10 +935,31 @@ var_hrswrun(struct variable *vp,
 #elif defined(sunos4)
 	    long_return = proc_table[LowProcIndex].p_time;
 #elif defined(cygwin)
-	    long_return = lowproc.rusage_self.ru_utime.tv_sec*100 +
-			  lowproc.rusage_self.ru_utime.tv_usec/10000 +
-	    		  lowproc.rusage_self.ru_stime.tv_sec*100 +
-			  lowproc.rusage_self.ru_stime.tv_usec/10000;
+	    {
+		DWORD n = lowproc.dwProcessId;
+		HANDLE h = OpenProcess(PROCESS_QUERY_INFORMATION | PROCESS_VM_READ,
+				       FALSE, n);
+		FILETIME ct, et, kt, ut;
+
+		if (h) {
+		    if (GetProcessTimes(h, &ct, &et, &kt, &ut))
+			long_return = (to_msec(&kt) + to_msec(&ut))/10;
+		    else {
+		        snmp_log(LOG_INFO, "no process times for %lu (%lu)\n", lowproc.pid, n);
+			long_return = 0;
+		    }
+		    CloseHandle(h);
+		}
+		else {
+		    snmp_log(LOG_INFO, "no process handle for %lu (%lu)\n", lowproc.pid, n);
+		    long_return = 0;
+		}
+	    }
+#elif defined(aix4)
+	    long_return = proc_table[LowProcIndex].p_ru.ru_utime.tv_sec*100 +
+			  proc_table[LowProcIndex].p_ru.ru_utime.tv_usec/10000 +
+	    		  proc_table[LowProcIndex].p_ru.ru_stime.tv_sec*100 +
+			  proc_table[LowProcIndex].p_ru.ru_stime.tv_usec/10000;
 #else
 	    long_return = proc_table[LowProcIndex].p_utime.tv_sec*100 +
 			  proc_table[LowProcIndex].p_utime.tv_usec/10000 +
@@ -713,6 +970,8 @@ var_hrswrun(struct variable *vp,
 	case HRSWRUNPERF_MEM:
 #ifdef HAVE_SYS_PSTAT_H
 	    long_return = (proc_buf.pst_rssize << PGSHIFT)/1024;
+#elif defined(dynix)
+            long_return = (lowpsinfo.pr_rssize * MMU_PAGESIZE)/1024;
 #elif defined(solaris2)
 #if _SLASH_PROC_METHOD_
 	    long_return = proc_buf ? proc_buf->pr_rssize : 0;
@@ -720,7 +979,7 @@ var_hrswrun(struct variable *vp,
 	    long_return = proc_buf->p_swrss;
 #endif
 #elif HAVE_KVM_GETPROCS
-#ifdef freebsd3
+#if defined(freebsd3) && !defined(darwin)
 	    long_return = proc_table[LowProcIndex].kp_eproc.e_vm.vm_map.size/1024;
 #else
 	    long_return = proc_table[LowProcIndex].kp_eproc.e_vm.vm_tsize +
@@ -741,14 +1000,26 @@ var_hrswrun(struct variable *vp,
 	    long_return = atoi( cp ) * (getpagesize()/1024);		/* rss */
             fclose(fp);
 #elif defined(cygwin)
-	    DEBUGMSGTL(("snmpd",
-"runPerfMem %d: %ld %ld %ld %ld %ld\n", lowproc.pid,
-lowproc.rusage_self.ru_maxrss,
-lowproc.rusage_self.ru_ixrss,
-lowproc.rusage_self.ru_idrss,
-lowproc.rusage_self.ru_isrss,
-lowproc.rusage_self.ru_nswap));
-	    long_return = lowproc.rusage_self.ru_idrss; /* ??? */
+	    {
+		DWORD n = lowproc.dwProcessId;
+		HANDLE h = OpenProcess(PROCESS_QUERY_INFORMATION | PROCESS_VM_READ,
+				       FALSE, n);
+		PROCESS_MEMORY_COUNTERS pmc;
+
+		if (h) {
+		    if (myGetProcessMemoryInfo && myGetProcessMemoryInfo(h, &pmc, sizeof pmc))
+			long_return = pmc.WorkingSetSize / 1024;
+		    else {
+		        snmp_log(LOG_INFO, "no process times for %lu (%lu)\n", lowproc.pid, n);
+			long_return = 0;
+		    }
+		    CloseHandle(h);
+		}
+		else {
+		    snmp_log(LOG_INFO, "no process handle for %lu (%lu)\n", lowproc.pid, n);
+		    long_return = 0;
+		}
+	    }
 #else
 #if NO_DUMMY_VALUES
 	    return NULL;
@@ -806,7 +1077,7 @@ End_HR_SWRun (void)
 
 #elif defined(cygwin)
 
-static int curpid;
+static pid_t curpid;
 
 void
 Init_HR_SWRun(void)
@@ -818,8 +1089,12 @@ Init_HR_SWRun(void)
 int
 Get_Next_HR_SWRun(void)
 {
-    curproc = (struct external_pinfo *) cygwin_internal(CW_GETPINFO_FULL, curpid|CW_NEXTPID);
-    curpid = curproc ? curproc->pid : -1;
+    curproc = (struct external_pinfo *) cygwin_internal(query, curpid|CW_NEXTPID);
+    if (curproc)
+	curpid = curproc->pid;
+    else {
+	curpid = -1;
+    }
     return curpid;
 }
 
@@ -829,6 +1104,35 @@ End_HR_SWRun(void)
     cygwin_internal(CW_UNLOCK_PINFO);
 }
 
+#elif defined(dynix)
+
+void
+Init_HR_SWRun(void)
+{
+    nextproc = 0;
+}
+
+int
+Get_Next_HR_SWRun(void)
+{
+    getprpsinfo_t *select = 0;
+
+    DEBUGMSGTL(("host/hr_swrun::GetNextHR_SWRun", "nextproc == %d... &nextproc = %u\n", nextproc, &nextproc));
+    if ( (nextproc = getprpsinfo(nextproc, select, &mypsinfo)) < 0 ) {
+        return -1;
+    } else {
+        DEBUGMSGTL(("host/hr_swrun::GetNextHR_SWRun", "getprpsinfo returned %d\n", nextproc));
+        return mypsinfo.pr_pid;
+    }
+
+}
+
+void
+End_HR_SWRun (void)
+{
+    /* just a stub... because it's declared */
+}
+
 #else /* linux */
 
 static int nproc;
@@ -847,7 +1151,7 @@ Init_HR_SWRun (void)
     }
     iwhen = now;
 
-#if defined(hpux10)
+#if defined(hpux10) || defined(hpux11)
     pstat_getdynamic( &pst_dyn, sizeof( struct pst_dynamic ),
 			1, 0 );
     nproc = pst_dyn.psd_activeprocs ;
@@ -933,7 +1237,7 @@ int
 Get_Next_HR_SWRun (void)
 {
     while ( current_proc_entry < nproc ) {
-#ifdef hpux10
+#if defined(hpux10) || defined(hpux11)
 	return proc_table[current_proc_entry++].pst_pid;
 #elif defined(solaris2)
 	return proc_table[current_proc_entry++];
@@ -948,7 +1252,6 @@ Get_Next_HR_SWRun (void)
 #endif
 
     }
-    End_HR_SWRun();
     return -1;
 }
 
@@ -961,16 +1264,16 @@ End_HR_SWRun (void)
 
 int count_processes (void)
 {
-#if !(defined(linux) || defined(cygwin) || defined(hpux10) || defined(solaris2) || HAVE_KVM_GETPROCS)
+#if !(defined(linux) || defined(cygwin)) || defined(hpux10) || defined(hpux11) || defined(solaris2) || HAVE_KVM_GETPROCS
     int i;
 #endif
     int total=0;
 
     Init_HR_SWRun();
-#if defined(hpux10) || HAVE_KVM_GETPROCS || defined(solaris2)
+#if defined(hpux10) || defined(hpux11) || HAVE_KVM_GETPROCS || defined(solaris2)
     total = nproc;
 #else
-#if !defined(linux) && !defined(cygwin)
+#if !defined(linux) && !defined(cygwin) && !defined(dynix)
     for ( i = 0 ; i<nproc ; ++i ) {
 	if ( proc_table[i].p_stat != 0 )
 #else
@@ -978,7 +1281,7 @@ int count_processes (void)
 #endif
 	    ++total;
     }
-#endif /* !hpux10 */
+#endif	/* !hpux10 && !hpux11 && !HAVE_KVM_GETPROCS && !solaris2 */
     End_HR_SWRun();
     return total;
 }
