@@ -38,6 +38,14 @@
 #endif
 #endif
 
+#ifdef solaris2
+#include <sys/var.h>
+#endif
+
+#if defined(hpux10) || defined(hpux11)
+#include <sys/pstat.h>
+#endif
+
 #ifdef HAVE_SYS_SYSCTL_H
 #include <sys/sysctl.h>
 #endif
@@ -157,7 +165,7 @@ var_hrsys(struct variable *vp,
 {
     static char string[100];
     time_t	now;
-#ifndef NR_TASKS
+#if !defined(NR_TASKS) && !defined(solaris2) && !defined(hpux10) && !defined(hpux11)
     int		nproc = 0;
 #endif
 #ifdef linux
@@ -167,6 +175,12 @@ var_hrsys(struct variable *vp,
     static int maxproc_mib[] = { CTL_KERN, KERN_MAXPROC };
     int buf_size;
 #endif
+#ifdef solaris2
+    struct var v;
+#endif
+#if defined(hpux10) || defined(hpux11)
+    struct pst_static pst_buf;
+#endif
 
     if (header_hrsys(vp, name, length, exact, var_len, write_method) == MATCH_FAILED )
 	return NULL;
@@ -215,9 +229,15 @@ var_hrsys(struct variable *vp,
 	    if (sysctl(maxproc_mib, 2, &nproc, &buf_size, NULL, 0) < 0)
 		    return NULL;
 	    long_return = nproc;
+#elif defined(hpux10) || defined(hpux11)
+	    pstat_getstatic(&pst_buf, sizeof(struct pst_static), 1, 0);
+	    long_return = pst_buf.max_proc;
 #elif defined(NPROC_SYMBOL)
 	    auto_nlist(NPROC_SYMBOL, (char *)&nproc, sizeof (int));
 	    long_return = nproc;
+#elif defined(solaris2)
+	    getKstatRaw("unix", "var", sizeof v, &v);
+	    long_return = v.v_proc;
 #else
 #if NO_DUMMY_VALUES
 	    return NULL;
