@@ -18,7 +18,11 @@
 #endif
 #include <sys/types.h>
 #if TIME_WITH_SYS_TIME
+# ifdef WIN32
+#  include <sys/timeb.h>
+# else
 # include <sys/time.h>
+# endif
 # include <time.h>
 #else
 # if HAVE_SYS_TIME_H
@@ -57,7 +61,11 @@
 #include <sys/sysctl.h>
 #endif
 #if HAVE_NET_IF_DL_H
+#ifndef dynix
 #include <net/if_dl.h>
+#else
+#include <sys/net/if_dl.h>
+#endif
 #endif
 #if HAVE_SYS_STREAM_H
 #include <sys/stream.h>
@@ -68,7 +76,6 @@
 #ifdef solaris2
 #include "kernel_sunos5.h"
 #endif
-
 #if HAVE_WINSOCK_H
 #include <winsock.h>
 #endif
@@ -76,6 +83,11 @@
 #include <dmalloc.h>
 #endif
 
+#ifdef hpux11
+#include <sys/mib.h>
+#include <netinet/mib_kern.h>
+#endif	/* hpux11 */
+
 #include "mibincl.h"
 #include "at.h"
 #include "interfaces.h"
@@ -86,6 +98,11 @@
 # if defined(RTF_LLINFO) && !defined(irix6)
 #  define CAN_USE_SYSCTL 1
 # endif
+#endif
+
+#ifdef cygwin
+#define WIN32
+#include <windows.h>
 #endif
 
 	/*********************
@@ -95,7 +112,6 @@
 	 *
 	 *********************/
 
-
 #ifndef WIN32
 #ifndef solaris2
 static void ARP_Scan_Init (void);
@@ -107,7 +123,6 @@ static int ARP_Scan_Next (u_long *, char *, u_long *);
 #endif
 #endif
 
-
 	/*********************
 	 *
 	 *  Public interface functions
@@ -132,7 +147,6 @@ void init_at(void)
   REGISTER_MIB("mibII/at", at_variables, variable4, at_variables_oid);
 }
 
-
 #ifndef WIN32
 #ifndef solaris2
 
@@ -289,7 +303,6 @@ var_atEntry(struct variable *vp,
    }
    return NULL;
 }
-
 #else          /* solaris2 */
 
 typedef struct if_ip {
@@ -437,6 +450,8 @@ static char *at = 0;
 static struct arphd *at=0;
 static struct arptab *at_ptr, at_entry;
 static struct arpcom  at_com;
+#elif defined(hpux11)
+static mib_ipNetToMediaEnt *at = (mib_ipNetToMediaEnt *)0;
 #else
 static struct arptab *at=0;
 #endif
@@ -446,6 +461,43 @@ static void ARP_Scan_Init (void)
 {
 #ifndef CAN_USE_SYSCTL
 #ifndef linux
+#ifdef hpux11
+
+	int fd;
+	struct nmparms p;
+	int val;
+	unsigned int ulen;
+	int ret;
+
+	if (at) free(at);
+	at = (mib_ipNetToMediaEnt *)0;
+	arptab_size = 0;
+
+	if ((fd = open_mib("/dev/ip", O_RDONLY, 0, NM_ASYNC_OFF)) >= 0) {
+	    p.objid = ID_ipNetToMediaTableNum;
+	    p.buffer = (void *)&val;
+	    ulen = sizeof(int);
+	    p.len = &ulen;
+	    if ((ret = get_mib_info(fd, &p)) == 0)
+		arptab_size = val;
+
+	    if (arptab_size > 0) {
+		ulen = (unsigned)arptab_size * sizeof(mib_ipNetToMediaEnt);
+		at = (mib_ipNetToMediaEnt *)malloc(ulen);
+		p.objid = ID_ipNetToMediaTable;
+		p.buffer = (void *)at;
+		p.len = &ulen;
+		if ((ret = get_mib_info(fd, &p)) < 0)
+		    arptab_size = 0;
+	    }
+
+	    close_mib(fd);
+	}
+
+	arptab_current = 0;
+
+#else	/* hpux11 */
+
 	if (!at) {
 	    auto_nlist(ARPTAB_SIZE_SYMBOL, (char *)&arptab_size, sizeof arptab_size);
 #ifdef STRUCT_ARPHD_HAS_AT_NEXT
@@ -462,7 +514,10 @@ static void ARP_Scan_Init (void)
         auto_nlist(ARPTAB_SYMBOL, (char *)at, arptab_size * sizeof(struct arptab));
 #endif
 	arptab_current = 0;
-#else /* linux */
+
+#endif	/* hpux11 */
+#else	/* linux */
+
 	FILE *in = fopen ("/proc/net/arp", "r");
 	int i, n = 0;
         char line [128];
@@ -510,8 +565,10 @@ static void ARP_Scan_Init (void)
 		at [i].if_index = Interface_Index_By_Name(ifname, strlen(ifname));
 	}
 	fclose (in);
+
 #endif /* linux */
 #else /* CAN_USE_SYSCTL */
+
 	int mib[6];
 	size_t needed;
 
@@ -532,6 +589,7 @@ static void ARP_Scan_Init (void)
 		snmp_log_perror("actual retrieval of routing table");
 	lim = at + needed;
 	rtnext = at;
+
 #endif /* CAN_USE_SYSCTL */
 }
 
@@ -557,10 +615,20 @@ static int ARP_Scan_Next(u_long *IPAddr, char *PhysAddr, u_long *ifType)
 		/* return success */
 		return( 1 );
 	}
-#endif /* linux */
-  return 0; /* we need someone with an irix box to fix this section */
-#else
-#if !defined(ARP_SCAN_FOUR_ARGUMENTS) || defined(hpux)
+#elif defined(hpux11)
+	if (arptab_current < arptab_size) {
+	    /* copy values */
+	    *IPAddr = at[arptab_current].NetAddr;
+	    memcpy(PhysAddr, at[arptab_current].PhysAddr.o_bytes,
+		at[arptab_current].PhysAddr.o_length);
+	    *ifType = at[arptab_current].Type;
+	    *ifIndex = at[arptab_current].IfIndex;
+	    /* increment to point next entry */
+	    arptab_current++;
+	    /* return success */
+	    return (1);
+	}
+#elif !defined(ARP_SCAN_FOUR_ARGUMENTS) || defined(hpux)
 	register struct arptab *atab;
 
 	while (arptab_current < arptab_size) {
@@ -596,7 +664,11 @@ static int ARP_Scan_Next(u_long *IPAddr, char *PhysAddr, u_long *ifType)
 #endif
 	return(1);
 	}
-#else /* !defined(ARP_SCAN_FOUR_ARGUMENTS) || defined(hpux) */
+#endif	/* linux || hpux11 || !ARP_SCAN_FOUR_ARGUMENTS || hpux */
+
+  return 0; /* we need someone with an irix box to fix this section */
+
+#else	/* !CAN_USE_SYSCTL */
 	struct rt_msghdr *rtm;
 	struct sockaddr_inarp *sin;
 	struct sockaddr_dl *sdl;
@@ -614,13 +686,18 @@ static int ARP_Scan_Next(u_long *IPAddr, char *PhysAddr, u_long *ifType)
 			return(1);
 		}
 	}
-#endif /* !defined(ARP_SCAN_FOUR_ARGUMENTS) || defined(hpux) */
 	return(0);	    /* "EOF" */
 #endif /* !CAN_USE_SYSCTL */
 }
 #endif /* solaris2 */
+
 #else /* WIN32 */
 #include <iphlpapi.h>
+
+extern WriteMethod write_arp;
+MIB_IPNETROW *arp_row = NULL;
+int create_flag = 0;
+
 u_char *
 var_atEntry(struct variable *vp,
             oid *name,
@@ -631,12 +708,12 @@ var_atEntry(struct variable *vp,
 {
     /*
      * Address Translation table object identifier is of form:
-     * 1.3.6.1.2.1.3.1.1.1.interface.1.A.B.C.D,  where A.B.C.D is IP address.
+     * 1.3.6.1.2.1.3.1.?.interface.1.A.B.C.D,  where A.B.C.D is IP address.
      * Interface is at offset 10,
      * IPADDR starts at offset 12.
      *
      * IP Net to Media table object identifier is of form:
-     * 1.3.6.1.2.1.4.22.1.1.1.interface.A.B.C.D,  where A.B.C.D is IP address.
+     * 1.3.6.1.2.1.4.22.1.?.interface.A.B.C.D,  where A.B.C.D is IP address.
      * Interface is at offset 10,
      * IPADDR starts at offset 11.
      */
@@ -650,6 +727,7 @@ var_atEntry(struct variable *vp,
     DWORD status = NO_ERROR;
     DWORD dwActualSize = 0;
     UINT i;
+    u_char dest_addr[4];
 
     /* fill in object part of name for current (less sizeof instance part) */
     memcpy((char *)current, (char *)vp->name, (int)vp->namelen * sizeof(oid));
@@ -670,13 +748,7 @@ var_atEntry(struct variable *vp,
             status = GetIpNetTable(pIpNetTable, &dwActualSize, TRUE);
         }
     }
-    if (current[6] == 3 ) {        /* AT group oid */
-        current[11] = 1;
-        op = current + 12;
-    }
-    else {                        /* IP NetToMedia group oid */
-        op = current + 11;
-    }
+
 
     if(status == NO_ERROR)
     {
@@ -684,6 +756,14 @@ var_atEntry(struct variable *vp,
         {
             current[10] =  pIpNetTable->table[i].dwIndex;
             
+            
+            if (current[6] == 3 ) {        /* AT group oid */
+                current[11] = 1;
+                op = current + 12;
+            }
+            else {                        /* IP NetToMedia group oid */
+                op = current + 11;
+            }
             cp = (u_char *)&pIpNetTable->table[i].dwAddr;
             *op++ = *cp++;
             *op++ = *cp++;
@@ -705,13 +785,45 @@ var_atEntry(struct variable *vp,
             }
         }
     }
-    if (lowState < 0){
+    if(arp_row == NULL){
+     /* Free allocated memory in case of SET request's FREE phase */
+      arp_row = (PMIB_IPNETROW)malloc(sizeof(MIB_IPNETROW));
+    }
+
+    if (lowState < 0 || status != NO_ERROR){
+      /* for creation of new row, only ipNetToMediaTable case is considered */
+      if(*length == 15 || *length == 16){
+        create_flag = 1; 
+        *write_method = write_arp; 
+        arp_row->dwIndex = name[10];
+
+        if(*length == 15)/* ipNetToMediaTable */
+        {
+          i = 11;
+        }else /* at Table */
+        {
+          i = 12;
+        }
+       
+        dest_addr[0] = (u_char)name[i];
+        dest_addr[1] = (u_char)name[i+1];
+        dest_addr[2] = (u_char)name[i+2];
+        dest_addr[3] = (u_char) name[i+3];
+        arp_row->dwAddr = *((DWORD *)dest_addr);
+
+        arp_row->dwType = 4; /* Static */
+        arp_row->dwPhysAddrLen = 0;        
+      }      
         free(pIpNetTable);
         return(NULL);
     }
-    memcpy( (char *)name,(char *)lowest, oid_length * sizeof(oid));
+    
+    create_flag = 0;
+    memcpy((char *)name,(char *)lowest, oid_length * sizeof(oid));
     *length = oid_length;
-    *write_method = 0;
+    *write_method = write_arp;
+    *arp_row = pIpNetTable->table[i];
+
     switch(vp->magic){
         case IPMEDIAIFINDEX:                        /* also ATIFINDEX */
             *var_len = sizeof long_return;
@@ -738,4 +850,226 @@ var_atEntry(struct variable *vp,
    }
    return NULL;
 }
-#endif
+
+int
+write_arp(
+   int      action,
+   u_char   *var_val,
+   u_char   var_val_type,
+   size_t   var_val_len,
+   u_char   *statP,
+   oid      *name,
+   size_t   length)
+{
+  int var, retval = SNMP_ERR_NOERROR ;
+  static PMIB_IPNETROW oldarp_row = NULL;
+  MIB_IPNETROW temp_row;
+  DWORD status = NO_ERROR;
+
+    /*
+     * IP Net to Media table object identifier is of form:
+     * 1.3.6.1.2.1.4.22.1.?.interface.A.B.C.D,  where A.B.C.D is IP address.
+     * Interface is at offset 10,
+     * IPADDR starts at offset 11.
+     */
+
+    if (name[6] == 3 ) {	/* AT group oid */
+	if (length != 16) {
+          snmp_log(LOG_ERR, "length error\n");
+          return SNMP_ERR_NOCREATION;
+        }
+    }
+    else {			/* IP NetToMedia group oid */
+	if (length != 15) {
+           snmp_log(LOG_ERR, "length error\n");
+           return SNMP_ERR_NOCREATION;
+        }
+    }
+  
+
+  /* #define for ipNetToMediaTable entries are 1 less than corresponding sub-id in MIB
+   * i.e. IPMEDIAIFINDEX defined as 0, but ipNetToMediaIfIndex registered as 1
+   */
+  var = name[9] - 1;
+  switch(action) {
+    case RESERVE1:    
+      switch(var){
+        case IPMEDIAIFINDEX: 
+          if (var_val_type != ASN_INTEGER){
+            snmp_log(LOG_ERR, "not integer\n");
+            return SNMP_ERR_WRONGTYPE;
+          }
+          if((*((int *)var_val)) < 0 ){
+            snmp_log(LOG_ERR, "invalid media ifIndex");
+            return SNMP_ERR_WRONGVALUE;
+          }
+          if (var_val_len > sizeof(int)){
+            snmp_log(LOG_ERR, "bad length\n");
+            return SNMP_ERR_WRONGLENGTH;
+          }
+          break;
+        case IPMEDIANETADDRESS:
+          if (var_val_type != ASN_IPADDRESS){
+            snmp_log(LOG_ERR, "not IP Address\n");
+            return SNMP_ERR_WRONGTYPE;
+          }
+          if((*((int *)var_val)) < 0 ){
+            snmp_log(LOG_ERR, "invalid media net address");
+            return SNMP_ERR_WRONGVALUE;
+          }
+          if (var_val_len > sizeof(DWORD)){
+            snmp_log(LOG_ERR, "bad length\n");
+            return SNMP_ERR_WRONGLENGTH;
+          }
+          break;
+        case IPMEDIATYPE: 
+          if (var_val_type != ASN_INTEGER){
+            snmp_log(LOG_ERR, "not integer\n");
+            return SNMP_ERR_WRONGTYPE;
+          }
+          if((*((int *)var_val)) < 1 || (*((int *)var_val)) > 4 ){
+            snmp_log(LOG_ERR, "invalid media type");
+            return SNMP_ERR_WRONGVALUE;
+          }
+          if (var_val_len > sizeof(int)){
+            snmp_log(LOG_ERR, "bad length\n");
+            return SNMP_ERR_WRONGLENGTH;
+          }
+          break;
+        case IPMEDIAPHYSADDRESS:
+          if (var_val_type != ASN_OCTET_STR) {
+            snmp_log(LOG_ERR, "not octet str");
+            return SNMP_ERR_WRONGTYPE;
+          }
+          if (var_val_len != 6) {
+            snmp_log(LOG_ERR, "not correct ipAddress length: %d", var_val_len);
+            return SNMP_ERR_WRONGLENGTH;
+          }
+          break;
+        default:
+          DEBUGMSGTL(("snmpd", "unknown sub-id %d in write_rte\n", var+1));
+        	return SNMP_ERR_NOTWRITABLE;
+      }
+      break;
+    case RESERVE2:
+      /* Save the old value, in case of UNDO */
+      if(oldarp_row == NULL){
+        oldarp_row = (PMIB_IPNETROW) malloc(sizeof(MIB_IPNETROW));
+        *oldarp_row = *arp_row;
+      }
+      break;
+    case  ACTION:	/* Perform the SET action (if reversible) */
+      switch(var){
+        
+        case IPMEDIAIFINDEX: 
+          temp_row = *arp_row;
+          arp_row->dwIndex  = *((int *)var_val) ;
+          /* In case of new entry, physical address is mandatory.
+           * SetIpNetEntry will be done in COMMIT case 
+          */
+          if(!create_flag){
+            if(SetIpNetEntry(arp_row) != NO_ERROR){
+              arp_row->dwIndex = temp_row.dwIndex;
+              retval = SNMP_ERR_COMMITFAILED;
+            }
+	    /*  Don't know yet, whether change in ifIndex creates new
+		row or not.  */
+	    /*  else {  */
+	    /*    temp_row.dwType = 2;  */
+	    /*      if (SetIpNetEntry(&temp_row) != NO_ERROR) {  */
+	    /*        retval = SNMP_ERR_COMMITFAILED;  */
+	    /*      }  */
+	    /*  }  */
+          }
+          break;
+        case IPMEDIANETADDRESS:
+          temp_row = *arp_row;
+          arp_row->dwAddr  = *((int *)var_val);
+          if(!create_flag){
+            if(SetIpNetEntry(arp_row) != NO_ERROR){
+              arp_row->dwAddr = oldarp_row->dwAddr;
+              retval = SNMP_ERR_COMMITFAILED;
+            }else{
+              temp_row.dwType = 2;
+              if(SetIpNetEntry(&temp_row) != NO_ERROR){
+                snmp_log(LOG_ERR, "Failed in ACTION, while deleting old row \n");
+                retval = SNMP_ERR_COMMITFAILED;
+              }
+            } 
+          }
+          break;
+        case IPMEDIATYPE:
+          arp_row->dwType  = *((int *)var_val);
+          if(!create_flag){
+            if(SetIpNetEntry(arp_row) != NO_ERROR)
+              retval = SNMP_ERR_COMMITFAILED;   
+          }
+          break;
+        case IPMEDIAPHYSADDRESS:
+          memcpy(arp_row->bPhysAddr, var_val, var_val_len);
+          arp_row->dwPhysAddrLen = var_val_len;
+          if(!create_flag){
+            if(SetIpNetEntry(arp_row) != NO_ERROR)
+              retval = SNMP_ERR_COMMITFAILED;
+          }
+          break;
+        default:
+          DEBUGMSGTL(("snmpd", "unknown sub-id %d in write_arp\n", var+1));
+        	retval = SNMP_ERR_NOTWRITABLE;
+      }
+      return retval;
+    case UNDO :
+      /* Reverse the SET action and free resources */            
+      if(oldarp_row != NULL){   
+        /*  UNDO the changes done for existing entry.  */
+        if(!create_flag){
+          if((status = SetIpNetEntry(oldarp_row)) != NO_ERROR){
+            snmp_log(LOG_ERR, "Error in case UNDO, status : %d\n", status);
+            retval = SNMP_ERR_UNDOFAILED;
+          } 
+        }
+
+        if(oldarp_row->dwAddr != arp_row->dwAddr){
+          arp_row->dwType = 2; /*  If row was added/created delete that row  */
+          
+          if((status = SetIpNetEntry(arp_row)) != NO_ERROR){
+            snmp_log(LOG_ERR, "Error while deleting added row, status : %d\n", 
+                   status);            
+            retval = SNMP_ERR_UNDOFAILED;
+          }
+        } 
+        free(oldarp_row);
+        oldarp_row = NULL;
+        free(arp_row);
+        arp_row = NULL; 
+        return retval;
+      }
+      break;
+    case COMMIT:
+      /*  if new entry and physical address specified, create new entry */
+      if(create_flag) {
+        if(arp_row->dwPhysAddrLen != 0){
+          if((status =CreateIpNetEntry(arp_row) )!= NO_ERROR){
+            snmp_log(LOG_ERR, "Inside COMMIT: CreateIpNetEntry failed, status %d\n", status);
+            retval = SNMP_ERR_COMMITFAILED;
+          }          
+        }else{
+          /*For new entry, physical address must be set. */
+          snmp_log(LOG_ERR, "Can't create new entry without physical address\n");
+          retval = SNMP_ERR_WRONGVALUE;
+        }
+        /* unset the create_flag, so that CreateIpNetEntry called only once */
+        create_flag = 0;
+      }
+                
+    case FREE:       
+      /* Free any resources allocated */     
+      free(oldarp_row);
+      oldarp_row = NULL;
+      free(arp_row);
+      arp_row = NULL;            
+      break;
+  }
+  return retval;
+}
+#endif /* WIN32 */
