@@ -15,6 +15,7 @@
 #else
 #include <strings.h>
 #endif
+
 #if HAVE_STDLIB_H
 #include <stdlib.h>
 #endif
@@ -35,15 +36,21 @@
 #if HAVE_SYS_SOCKET_H
 #include <sys/socket.h>
 #endif
+#ifndef STREAM_NEEDS_KERNEL_ISLANDS
 #if HAVE_SYS_STREAM_H
 #include <sys/stream.h>
 #endif
+#endif
 #if HAVE_SYS_SOCKETVAR_H
 #include <sys/socketvar.h>
 #endif
 
 #if TIME_WITH_SYS_TIME
+# ifdef WIN32
+#  include <sys/timeb.h>
+# else
 # include <sys/time.h>
+# endif
 # include <time.h>
 #else
 # if HAVE_SYS_TIME_H
@@ -73,6 +80,11 @@
 #ifdef _I_DEFINED_KERNEL
 #undef _KERNEL
 #endif
+#ifdef STREAM_NEEDS_KERNEL_ISLANDS
+#if HAVE_SYS_STREAM_H
+#include <sys/stream.h>
+#endif
+#endif
 #if HAVE_NET_ROUTE_H
 #include <net/route.h>
 #endif
@@ -114,7 +126,11 @@
 #include <net/if_types.h>
 #endif
 #if HAVE_NET_IF_DL_H
+#ifndef dynix
 #include <net/if_dl.h>
+#else
+#include <sys/net/if_dl.h>
+#endif
 #endif
 #if HAVE_INET_MIB2_H
 #include <inet/mib2.h>
@@ -140,10 +156,15 @@
 #include <netinet/mib_kern.h>
 #endif /* hpux */
 
+#ifdef cygwin
+#define WIN32
+#include <windows.h>
+#endif
+
 #if HAVE_SYS_SYSCTL_H
 #include <sys/sysctl.h>
 
-#ifdef freebsd3
+#if defined(freebsd3) || defined(freebsd4) || defined(freebsd5)
 #    define USE_SYSCTL_IFLIST
 #else
 # if defined(CTL_NET) && !defined(freebsd2)
@@ -191,7 +212,11 @@ struct variable4 interfaces_variables[] = {
     {IFMTU, ASN_INTEGER, RONLY, var_ifEntry, 3, {2, 1, 4}},
     {IFSPEED, ASN_GAUGE, RONLY, var_ifEntry, 3, {2, 1, 5}},
     {IFPHYSADDRESS, ASN_OCTET_STR, RONLY, var_ifEntry, 3, {2, 1, 6}},
+#ifdef WIN32
+    {IFADMINSTATUS, ASN_INTEGER, RWRITE, var_ifEntry, 3, {2, 1, 7}},
+#else
     {IFADMINSTATUS, ASN_INTEGER, RONLY, var_ifEntry, 3, {2, 1, 7}},
+#endif
     {IFOPERSTATUS, ASN_INTEGER, RONLY, var_ifEntry, 3, {2, 1, 8}},
     {IFLASTCHANGE, ASN_TIMETICKS, RONLY, var_ifEntry, 3, {2, 1, 9}},
     {IFINOCTETS, ASN_COUNTER, RONLY, var_ifEntry, 3, {2, 1, 10}},
@@ -232,6 +257,7 @@ void init_interfaces(void)
 #endif
 }
 
+#ifdef linux
 /*
  * if_type_from_name
  * Return interface type using the interface name as a clue.
@@ -266,6 +292,7 @@ if_type_from_name( const char *pcch)
     }
     return (1); /* in case search fails */
 }
+#endif
 
 
 typedef struct _conf_if_list {
@@ -519,12 +546,10 @@ Interface_Scan_By_Index (int iindex,
 	  DEBUGMSGTL(("mibII/interfaces", "routing socket: unknown message type %d\n", ifp->ifm_type));
 	}
     }
-  if (have_ifinfo && have_addr)
+  if (have_ifinfo)
     {
       return 0;
     }
-  else if (have_ifinfo && !(if_msg->ifm_flags & IFF_UP))
-      return 0;
   else
     {
       return -1;
@@ -728,8 +753,12 @@ int Interface_Scan_Next(short *Index,
 #ifndef HAVE_NET_IF_MIB_H
 
 #ifndef solaris2
+#ifndef hpux11
 static int Interface_Scan_By_Index (int, char *, struct ifnet *, struct in_ifaddr *);
 static int Interface_Get_Ether_By_Index (int, u_char *);
+#else
+static int Interface_Scan_By_Index (int, char *, nmapi_phystat *);
+#endif
 #endif
 
 
@@ -960,26 +989,43 @@ var_ifEntry(struct variable *vp,
 	    size_t *var_len,
 	    WriteMethod **write_method)
 {
+#if defined(hpux11)
+    static nmapi_phystat ifnet;
+#else
     static struct ifnet ifnet;
+#endif
     register int interface;
+#if !defined(hpux11)
     static struct in_ifaddr in_ifaddrVar;
+#endif
+#if defined(hpux11)
+    static char Name[MAX_PHYSADDR_LEN];
+#else
     static char Name[16];
+#endif
     register char *cp;
 #if STRUCT_IFNET_HAS_IF_LASTCHANGE_TV_SEC
           struct timeval now;
 #endif
+#if !defined(hpux11)
     struct nmparms hp_nmparms;
     static mib_ifEntry hp_ifEntry;
     int  hp_fd;
     int  hp_len=sizeof(hp_ifEntry);
+#endif
 
 
     interface = header_ifEntry(vp, name, length, exact, var_len, write_method);
     if ( interface == MATCH_FAILED )
 	return NULL;
 
+#if defined(hpux11)
+    Interface_Scan_By_Index(interface, Name, &ifnet);
+#else
     Interface_Scan_By_Index(interface, Name, &ifnet, &in_ifaddrVar);
+#endif
 
+#if !defined(hpux11)
 	/*
 	 * Additional information about the interfaces is available under
 	 * HP-UX through the network management interface '/dev/netman'
@@ -997,35 +1043,56 @@ var_ifEntry(struct variable *vp,
           hp_fd = -1;         /* failed */
       }
     }
+#endif
 
     switch (vp->magic){
 	case IFINDEX:
 	    long_return = interface;
 	    return (u_char *) &long_return;
 	case IFDESCR:
+#if defined(hpux11)
+	  cp = ifnet.if_entry.ifDescr;
+#else
 	  if ( hp_fd != -1 )
 	    cp = hp_ifEntry.ifDescr;
 	  else
 	    cp = Name;
+#endif
 	    *var_len = strlen(cp);
 	    return (u_char *)cp;
 	case IFTYPE:
+#if defined(hpux11)
+	    long_return = ifnet.if_entry.ifType;
+#else
 	      if ( hp_fd != -1 )
 		long_return = hp_ifEntry.ifType;
 	      else
 		long_return = 1;	/* OTHER */
+#endif
 	    return (u_char *) &long_return;
 	case IFMTU: {
+#if defined(hpux11)
+	    long_return = (long) ifnet.if_entry.ifMtu;
+#else
 	    long_return = (long) ifnet.if_mtu;
+#endif
 	    return (u_char *) &long_return;
 	}
 	case IFSPEED:
+#if defined(hpux11)
+	    long_return = ifnet.if_entry.ifSpeed;
+#else
 	    if ( hp_fd != -1 )
 		long_return = hp_ifEntry.ifSpeed;
 	    else
 	        long_return = (u_long)  1;	/* OTHER */
+#endif
 	    return (u_char *) &long_return;
 	case IFPHYSADDRESS:
+#if defined(hpux11)
+		*var_len = ifnet.if_entry.ifPhysAddress.o_length;
+		return (u_char *) ifnet.if_entry.ifPhysAddress.o_bytes;
+#else
 		Interface_Get_Ether_By_Index(interface, return_buf);
 		*var_len = 6;
 	        if ((return_buf[0] == 0) && (return_buf[1] == 0) &&
@@ -1033,74 +1100,139 @@ var_ifEntry(struct variable *vp,
 		    (return_buf[4] == 0) && (return_buf[5] == 0))
 		    *var_len = 0;
 		return(u_char *) return_buf;
+#endif
 	case IFADMINSTATUS:
+#if defined(hpux11)
+	    long_return = ifnet.if_entry.ifAdmin;
+#else
 	    long_return = ifnet.if_flags & IFF_UP ? 1 : 2;
+#endif
 	    return (u_char *) &long_return;
 	case IFOPERSTATUS:
+#if defined(hpux11)
+	    long_return = ifnet.if_entry.ifOper;
+#else
 	    long_return = ifnet.if_flags & IFF_RUNNING ? 1 : 2;
+#endif
 	    return (u_char *) &long_return;
 	case IFLASTCHANGE:
+#if defined(hpux11)
+	  long_return = ifnet.if_entry.ifLastChange;
+#else
 	  if ( hp_fd != -1 )
 	    long_return = hp_ifEntry.ifLastChange;
 	  else
           long_return = 0; /* XXX */
+#endif
           return (u_char *) &long_return;
 	case IFINOCTETS:
+#if defined(hpux11)
+	  long_return = ifnet.if_entry.ifInOctets;
+#else
 	  if ( hp_fd != -1 )
 	    long_return = hp_ifEntry.ifInOctets;
 	  else
 	    long_return = (u_long)  ifnet.if_ipackets * 308; /* XXX */
+#endif
 	  return (u_char *) &long_return;
 	case IFINUCASTPKTS:
+#if defined(hpux11)
+	  long_return = ifnet.if_entry.ifInUcastPkts;
+#else
 	  if ( hp_fd != -1 )
 	    long_return = hp_ifEntry.ifInUcastPkts;
 	  else
 	    long_return = (u_long)  ifnet.if_ipackets;
+#endif
 	  return (u_char *) &long_return;
 	case IFINNUCASTPKTS:
+#if defined(hpux11)
+	  long_return = ifnet.if_entry.ifInNUcastPkts;
+#else
 	  if ( hp_fd != -1 )
 	    long_return = hp_ifEntry.ifInNUcastPkts;
 	  else
 	    long_return = (u_long)  0; /* XXX */
+#endif
 	    return (u_char *) &long_return;
 	case IFINDISCARDS:
+#if defined(hpux11)
+	  long_return = ifnet.if_entry.ifInDiscards;
+#else
 	  if ( hp_fd != -1 )
 	    long_return = hp_ifEntry.ifInDiscards;
 	  else
 	    long_return = (u_long)  0; /* XXX */
+#endif
 	    return (u_char *) &long_return;
 	case IFINERRORS:
-	    return (u_char *) &ifnet.if_ierrors;
+#if defined(hpux11)
+	    long_return = ifnet.if_entry.ifInErrors;
+#else
+	    long_return = ifnet.if_ierrors;
+#endif
+	    return (u_char *) &long_return;
 	case IFINUNKNOWNPROTOS:
+#if defined(hpux11)
+	  long_return = ifnet.if_entry.ifInUnknownProtos;
+#else
 	  if ( hp_fd != -1 )
 	    long_return = hp_ifEntry.ifInUnknownProtos;
 	  else
 	    long_return = (u_long)  0; /* XXX */
+#endif
 	    return (u_char *) &long_return;
 	case IFOUTOCTETS:
+#if defined(hpux11)
+	  long_return = ifnet.if_entry.ifOutOctets;
+#else
 	  if ( hp_fd != -1 )
 	    long_return = hp_ifEntry.ifOutOctets;
 	  else
 	    long_return = (u_long)  ifnet.if_opackets * 308; /* XXX */
+#endif
 	    return (u_char *) &long_return;
 	case IFOUTUCASTPKTS:
+#if defined(hpux11)
+	  long_return = ifnet.if_entry.ifOutUcastPkts;
+#else
 	  if ( hp_fd != -1 )
 	    long_return = hp_ifEntry.ifOutUcastPkts;
 	  else
 	    long_return = (u_long)  ifnet.if_opackets;
+#endif
 	    return (u_char *) &long_return;
 	case IFOUTNUCASTPKTS:
+#if defined(hpux11)
+	  long_return = ifnet.if_entry.ifOutNUcastPkts;
+#else
 	  if ( hp_fd != -1 )
 	    long_return = hp_ifEntry.ifOutNUcastPkts;
 	  else
 	    long_return = (u_long)  0; /* XXX */
+#endif
 	    return (u_char *) &long_return;
 	case IFOUTDISCARDS:
-	    return (u_char *) &ifnet.if_snd.ifq_drops;
+#if defined(hpux11)
+	    long_return = ifnet.if_entry.ifOutDiscards;
+#else
+	    long_return = ifnet.if_snd.ifq_drops;
+#endif
+	    return (u_char *) &long_return;
 	case IFOUTERRORS:
-	    return (u_char *) &ifnet.if_oerrors;
+#if defined(hpux11)
+	    long_return = ifnet.if_entry.ifOutErrors;
+#else
+	    long_return = ifnet.if_oerrors;
+#endif
+	    return (u_char *) &long_return;
 	case IFOUTQLEN:
-	    return (u_char *) &ifnet.if_snd.ifq_len;
+#if defined(hpux11)
+	    long_return = ifnet.if_entry.ifOutQlen;
+#else
+	    long_return = ifnet.if_snd.ifq_len;
+#endif
+	    return (u_char *) &long_return;
 	case IFSPECIFIC:
 	    *var_len = nullOidLen;
 	    return (u_char *) nullOid;
@@ -1230,12 +1362,14 @@ var_ifEntry(struct variable *vp,
 
 #ifndef solaris2
 
-#if !defined(sunV3) && !defined(linux)
+#if !defined(sunV3) && !defined(linux) && !defined(hpux11)
 static struct in_ifaddr savein_ifaddr;
 #endif
+#if !defined(hpux11)
 static struct ifnet *ifnetaddr, saveifnet, *saveifnetaddr;
-static int saveIndex=0;
 static char saveName[16];
+#endif
+static int saveIndex=0;
 
 void
 Interface_Scan_Init (void)
@@ -1245,16 +1379,18 @@ Interface_Scan_Init (void)
     struct ifreq ifrq;
     struct ifnet **ifnetaddr_ptr;
     FILE *devin;
-    unsigned long rec_pkt, rec_oct, rec_err, snd_pkt, snd_oct, snd_err, coll;
+    unsigned long long rec_pkt, rec_oct, rec_err, snd_pkt, snd_oct, snd_err, coll;
     int i, fd;
     conf_if_list *if_ptr;
-    const char *scan_line_2_2="%lu %lu %lu %*lu %*lu %*lu %*lu %*lu %lu %lu %lu %*lu %*lu %lu";
-    const char *scan_line_2_0="%lu %lu %*lu %*lu %*lu %lu %lu %*lu %*lu %lu";
+    const char *scan_line_2_2="%llu %llu %llu %*llu %*llu %*llu %*llu %*llu %llu %llu %llu %*llu %*llu %llu";
+    const char *scan_line_2_0="%llu %llu %*llu %*llu %*llu %llu %llu %*llu %*llu %llu";
     const char *scan_line_to_use;
     
 #endif  
 
+#if !defined(hpux11)
     auto_nlist(IFNET_SYMBOL, (char *)&ifnetaddr, sizeof(ifnetaddr));
+#endif
     saveIndex=0;
 
 #ifdef linux
@@ -1565,13 +1701,50 @@ Interface_Index_By_Name(char *Name,
 #endif
 
 
-#else
+#else	/* sunV3 || linux */
 
 #if defined(netbsd1) || defined(openbsd2)
 #define ia_next ia_list.tqe_next
 #define if_next if_list.tqe_next
 #endif
 
+#if defined(hpux11)
+
+int Interface_Scan_Next(short *Index,
+			char *Name,
+			nmapi_phystat *Retifnet)
+{
+	static nmapi_phystat *if_ptr = (nmapi_phystat *)0;
+	int count = Interface_Scan_Get_Count();
+	unsigned int ulen;
+	int ret;
+
+	if (!if_ptr) {
+	    if (count) 
+		if_ptr = (nmapi_phystat *)malloc(sizeof(nmapi_phystat) * count);
+	    else
+		return (0);	/* EOF */
+	}
+
+	if (saveIndex >= count)
+	    return (0);		/* EOF */
+
+	ulen = (unsigned int)count * sizeof(nmapi_phystat);
+	if ((ret = get_physical_stat(if_ptr, &ulen)) < 0)
+	    return (0);	/* EOF */
+
+	if (Retifnet)
+	    *Retifnet = if_ptr[saveIndex];
+	if (Name)
+	    strcpy(Name, if_ptr[saveIndex].nm_device);
+	saveIndex++;
+	if (Index)
+	    *Index = saveIndex;
+	return (1);	/* DONE */
+}
+
+#else	/* hpux11 */
+
 int Interface_Scan_Next(short *Index,
 			char *Name,
 			struct ifnet *Retifnet,
@@ -1644,8 +1817,27 @@ int Interface_Scan_Next(short *Index,
 	return(0);	    /* EOF */
 }
 
+#endif	/* hpux11 */
 
-#endif /* sunV3 */
+#endif	/* sunV3 || linux */
+
+#if defined(hpux11)
+
+static int Interface_Scan_By_Index(int Index,
+				   char *Name,
+				   nmapi_phystat *Retifnet)
+{
+	short i;
+
+	Interface_Scan_Init();
+	while (Interface_Scan_Next(&i, Name, Retifnet)) {
+	    if (i == Index) break;
+	}
+	if (i != Index) return(-1);     /* Error, doesn't exist */
+	return(0);      /* DONE */
+}
+
+#else	/* hpux11 */
 
 static int Interface_Scan_By_Index(int Index,
 				   char *Name,
@@ -1662,8 +1854,36 @@ static int Interface_Scan_By_Index(int Index,
 	return(0);	/* DONE */
 }
 
+#endif	/* hpux11 */
 
 static int Interface_Count=0;
+
+#if defined(hpux11)
+
+static int Interface_Scan_Get_Count (void)
+{
+	if (!Interface_Count) {
+	    int fd;
+	    struct nmparms p;
+	    int val;
+	    unsigned int ulen;
+	    int ret;
+
+	    if ((fd = open_mib("/dev/ip", O_RDONLY, 0, NM_ASYNC_OFF)) >= 0) {
+		p.objid = ID_ifNumber;
+		p.buffer = (void *)&val;
+		ulen = sizeof(int);
+		p.len = &ulen;
+		if ((ret = get_mib_info(fd, &p)) == 0)
+		    Interface_Count = val;
+		close_mib(fd);
+	    }
+	}
+	return(Interface_Count);
+}
+
+#else	/* hpux11 */
+
 static time_t scan_time = 0;
 
 static int Interface_Scan_Get_Count (void)
@@ -1673,6 +1893,7 @@ static int Interface_Scan_Get_Count (void)
 	if (!Interface_Count || (time_now > scan_time + 60)) {
             scan_time = time_now;
 	    Interface_Scan_Init();
+	    Interface_Count = 0;
 	    while (Interface_Scan_Next(NULL, NULL, NULL, NULL) != 0) {
 		Interface_Count++;
 	    }
@@ -1773,7 +1994,9 @@ static int Interface_Get_Ether_By_Index(int Index,
 	return(0);	/* DONE */
 }
 
-#else /* solaris2 */
+#endif	/* hpux11 */
+
+#else	/* solaris2 */
 
 static int Interface_Scan_Get_Count (void)
 {
@@ -1897,10 +2120,18 @@ loop:
 			physaddrbuf = 0;
 		}
 		ifm = (struct if_msghdr *)rtm;
+#if defined(freebsd3) || defined(freebsd4) || defined(freebsd5)
+		if (physaddrs != 0) physaddrs[naddrs] = (void*)(ifm + 1);
+		naddrs++;
+#endif
 		ilen -= ifm->ifm_msglen;
 		cp += ifm->ifm_msglen;
 		rtm = (struct rt_msghdr *)cp;
 		while (ilen > 0 && rtm->rtm_type == RTM_NEWADDR) {
+#if defined(freebsd3) || defined(freebsd4) || defined(freebsd5)
+			ilen -= rtm->rtm_msglen;
+			cp += rtm->rtm_msglen;
+#else
 			int is_alias = 0;
 			ifam = (struct ifa_msghdr *)rtm;
 			ilen -= sizeof(*ifam);
@@ -1923,6 +2154,7 @@ loop:
 				}
 				sa = (struct sockaddr *)cp;
 			}
+#endif
 			rtm = (struct rt_msghdr *)cp;
 		}
 	}
@@ -2037,7 +2269,7 @@ var_ifEntry(struct variable *vp,
 		if (ifmd.ifmd_data.ifi_lastchange.tv_sec == 0 &&
 		    ifmd.ifmd_data.ifi_lastchange.tv_usec == 0) {
 			long_return = 0;
-		else if (ifmd.ifmd_data.ifi_lastchange.tv_sec < starttime.tv_sec)
+		} else if (ifmd.ifmd_data.ifi_lastchange.tv_sec < starttime.tv_sec) {
 		    long_return = 0;
 		} else {
 		    long_return = (u_long)
@@ -2093,10 +2325,15 @@ var_ifEntry(struct variable *vp,
 }
 
 #endif /* HAVE_NET_IF_MIB_H */
-#endif
+#endif /* !USE_SYSCTL_IFLIST */
 
 #else /* WIN32 */
 #include <iphlpapi.h>
+
+WriteMethod writeIfEntry;
+long admin_status = 0;
+long oldadmin_status = 0;
+
 static int
 header_ifEntry(struct variable *vp,
 	       oid *name,
@@ -2121,7 +2358,6 @@ header_ifEntry(struct variable *vp,
     memcpy( (char *)newname,(char *)vp->name, (int)vp->namelen * sizeof(oid));
     /* find "next" ifIndex */
     
-/* *****Check if caching is NECESSARY****************** */
 
     /* query for buffer size needed */
     status = GetIfTable(pIfTable, &dwActualSize, TRUE);
@@ -2162,6 +2398,8 @@ header_ifEntry(struct variable *vp,
     return count;
 }
 
+
+
 u_char *
 var_interfaces(struct variable *vp,
 	       oid *name,
@@ -2195,14 +2433,14 @@ var_ifEntry(struct variable *vp,
     int ifIndex;
     static char Name[16];
     conf_if_list *if_ptr = conf_list;
-    MIB_IFROW ifRow;
+    static MIB_IFROW ifRow;
 
     ifIndex = header_ifEntry(vp, name, length, exact, var_len, write_method);
     if ( ifIndex == MATCH_FAILED )
 	      return NULL;
 
-    ifRow.dwIndex = ifIndex;
     /* Get the If Table Row by passing index as argument */
+    ifRow.dwIndex = ifIndex;
     if(GetIfEntry(&ifRow) != NO_ERROR)
       return NULL;
     switch (vp->magic){
@@ -2227,6 +2465,8 @@ var_ifEntry(struct variable *vp,
 	    return(u_char *) return_buf;
 	case IFADMINSTATUS:
 	    long_return = ifRow.dwAdminStatus;
+      admin_status = long_return;
+      *write_method = writeIfEntry;
 	    return (u_char *) &long_return;
 	case IFOPERSTATUS:
 	    long_return = ifRow.dwOperStatus;
@@ -2279,5 +2519,70 @@ var_ifEntry(struct variable *vp,
     return NULL;
 }
 
+
+int
+writeIfEntry(int action,	     
+	    u_char *var_val,
+	    u_char var_val_type,
+	    size_t var_val_len,
+	    u_char *statP,
+	    oid *name,
+	    size_t name_len)
+{
+    MIB_IFROW ifEntryRow;
+    if((char)name[9] != IFADMINSTATUS){     
+	      return SNMP_ERR_NOTWRITABLE;		
+    }
+
+    switch ( action ) {
+	case RESERVE1:		/* Check values for acceptability */
+	    if (var_val_type != ASN_INTEGER){
+                snmp_log(LOG_ERR, "not integer\n");
+		     return SNMP_ERR_WRONGTYPE;
+	    }
+	    if (var_val_len > sizeof(int)){
+                snmp_log(LOG_ERR, "bad length\n");
+		     return SNMP_ERR_WRONGLENGTH;
+	    }
+	    
+      /* The dwAdminStatus member can be MIB_IF_ADMIN_STATUS_UP or MIB_IF_ADMIN_STATUS_DOWN */
+      if(!(((int)(*var_val) == MIB_IF_ADMIN_STATUS_UP) || 
+          ((int)(*var_val) == MIB_IF_ADMIN_STATUS_DOWN))){
+                    snmp_log(LOG_ERR, "not supported admin state\n");
+		    return SNMP_ERR_WRONGVALUE;
+		}
+	    break;
+
+	case RESERVE2:		/* Allocate memory and similar resources */
+	    break;
+
+	case ACTION:	
+		/* Save the old value, in case of UNDO */
+	    
+      oldadmin_status   = admin_status;
+      admin_status = (int)*var_val;
+	    break;
+
+	case UNDO:		/* Reverse the SET action and free resources */     
+	    admin_status = oldadmin_status;
+	    break;
+
+	case COMMIT:		/* Confirm the SET, performing any irreversible actions,
+					and free resources */    
+    ifEntryRow.dwIndex = (int)name[10];
+    ifEntryRow.dwAdminStatus = admin_status;
+    /* Only UP and DOWN status are supported. Thats why done in COMMIT */
+    if(SetIfEntry(&ifEntryRow) != NO_ERROR){
+      snmp_log(LOG_ERR, "Error in writeIfEntry case COMMIT with index: %d & adminStatus %d\n", 
+        ifEntryRow.dwIndex, ifEntryRow.dwAdminStatus);
+      return SNMP_ERR_COMMITFAILED;
+    }
+   
+	case FREE:		/* Free any resources allocated */
+		/* No resources have been allocated */
+	    break;
+    }
+    return SNMP_ERR_NOERROR;
+} /* end of writeIfEntry */
 #endif  /* WIN32 */
 
