@@ -14,6 +14,7 @@
 #if HAVE_WINSOCK_H
 #include <winsock.h>
 #endif
+
 #if HAVE_SYS_PARAM_H
 #include <sys/param.h>
 #endif
@@ -73,7 +74,9 @@
 #if HAVE_NETINET_IN_PCB_H
 #include <netinet/in_pcb.h>
 #endif
+#ifdef HAVE_NETINET_UDP_H
 #include <netinet/udp.h>
+#endif
 #if HAVE_NETINET_UDP_VAR_H
 #include <netinet/udp_var.h>
 #endif
@@ -86,15 +89,22 @@
 #endif
 
 #include "tools.h"
+
 #ifdef solaris2
 #include "kernel_sunos5.h"
 #else
 #include "kernel.h"
 #endif
+
 #ifdef linux
 #include "kernel_linux.h"
 #endif
 
+#ifdef cygwin
+#define WIN32
+#include <windows.h>
+#endif
+
 #include "mibincl.h"
 #include "util_funcs.h"
 #include "system.h"
@@ -190,6 +200,10 @@ void init_udp(void)
 #define USES_SNMP_DESIGNED_UDPSTAT
 #endif
 
+#ifdef hpux11
+#define UDP_STAT_STRUCTURE	int
+#endif
+
 #ifdef WIN32
 #include <iphlpapi.h>
 #define UDP_STAT_STRUCTURE MIB_UDPSTATS
@@ -200,6 +214,7 @@ void init_udp(void)
 #define USES_TRADITIONAL_UDPSTAT
 #endif
 
+
 #if !defined(UDP_STAT_STRUCTURE)
 #define UDP_STAT_STRUCTURE	struct udpstat
 #define USES_TRADITIONAL_UDPSTAT
@@ -285,7 +300,6 @@ var_udp(struct variable *vp,
 				return (u_char *) &long_return;
 
 #endif		/* USES_TRADITIONAL_UDPSTAT */
-
 #ifdef WIN32
        case UDPINDATAGRAMS:
     return (u_char *) &udpstat.dwInDatagrams;
@@ -307,14 +321,12 @@ var_udp(struct variable *vp,
 #endif
 }
 
-
 	/*********************
 	 *
 	 *  Internal implementation functions
 	 *
 	 *********************/
 
-
 long
 read_udp_stat( UDP_STAT_STRUCTURE *udpstat, int magic )
 {
@@ -326,6 +338,33 @@ read_udp_stat( UDP_STAT_STRUCTURE *udpstat, int magic )
 #ifdef solaris2
     static mib2_ip_t ipstat;
 #endif
+#ifdef hpux11
+    int fd;
+    struct nmparms p;
+    unsigned int ulen;
+
+    if ((fd = open_mib("/dev/ip", O_RDONLY, 0, NM_ASYNC_OFF)) < 0)
+	return (-1);	/* error */
+
+    switch (magic) {
+	case UDPINDATAGRAMS:	p.objid = ID_udpInDatagrams;		break;
+	case UDPNOPORTS:	p.objid = ID_udpNoPorts;		break;
+	case UDPOUTDATAGRAMS:	p.objid = ID_udpOutDatagrams;		break;
+	case UDPINERRORS:	p.objid = ID_udpInErrors;		break;
+	default:
+	    *udpstat = 0;
+	    close_mib(fd);
+	    return (0);
+    }
+
+    p.buffer = (void *)udpstat;
+    ulen = sizeof(UDP_STAT_STRUCTURE);
+    p.len = &ulen;
+    ret_value = get_mib_info(fd, &p);
+    close_mib(fd);
+
+    return (ret_value);	/* 0: ok, < 0: error */
+#else	/* hpux11 */
 
     if (  udp_stats_cache_marker &&
 	(!atime_ready( udp_stats_cache_marker, UDP_STATS_CACHE_TIMEOUT*1000 )))
@@ -378,4 +417,5 @@ read_udp_stat( UDP_STAT_STRUCTURE *udpstat, int magic )
 	udp_stats_cache_marker = NULL;
     }
     return ret_value;
+#endif	/* hpux11 */
 }
