@@ -5,21 +5,13 @@
 #ifndef _MIBGROUP_UDP_H
 #define _MIBGROUP_UDP_H
 
-#ifdef linux
-struct udp_mib
-{
- 	unsigned long	UdpInDatagrams;
- 	unsigned long	UdpNoPorts;
- 	unsigned long	UdpInErrors;
- 	unsigned long	UdpOutDatagrams;
-};
-#endif
 
+config_require(mibII/udpTable util_funcs)
 config_arch_require(solaris2, kernel_sunos5)
+config_arch_require(linux, mibII/kernel_linux)
 
 extern void	init_udp (void);
 extern FindVarMethod var_udp;
-extern FindVarMethod var_udpEntry;
 
 
 #define UDPINDATAGRAMS	    0
