@@ -9,7 +9,7 @@ config_require(mibII/ip util_funcs)
 config_arch_require(solaris2, kernel_sunos5)
 
 void init_var_route (void);
-#ifdef RTENTRY_4_4
+#if defined(RTENTRY_4_4) && !defined(hpux11)
 struct radix_node;
 void load_rtentries (struct radix_node *);
 #endif
