@@ -25,12 +25,6 @@
 #  include <time.h>
 # endif
 #endif
-#if HAVE_NETINET_IN_H
-#include <netinet/in.h>
-#endif
-#if HAVE_SYS_SOCKET_H
-#include <sys/socket.h>
-#endif
 
 #include "mib_module_config.h"
 
