@@ -165,8 +165,14 @@ smux_parse_peer_auth(const char *token, char *cptr)
 	cptr = skip_white(cptr);
 
         /* password */
-        if (cptr)
-          strcpy(aptr->sa_passwd, cptr);
+        if (cptr) {
+	    if (strlen(cptr) > SMUXMAXSTRLEN - 1) {
+		config_perror("password (third token) is too long");
+		free((char *)aptr);
+		return;
+	    }
+	    strcpy(aptr->sa_passwd, cptr);
+	}
         
 	Auths[nauths++] = aptr;
 }
@@ -671,7 +677,7 @@ smux_open_process(int fd, u_char *ptr, size_t *len, int *fail)
 		*fail = TRUE;
 		return((ptr += *len));
 	}
-	sprint_objid(oid_print, oid_name, oid_name_len);
+	snprint_objid(oid_print, sizeof(oid_print), oid_name, oid_name_len);
 
         if (snmp_get_do_debugging()) {
 	    DEBUGMSGTL (("smux","[smux_open_process] smux peer: %s\n", oid_print)); 
@@ -758,8 +764,6 @@ smux_auth_peer(oid *name, size_t namelen, char *passwd, int fd)
 				Auths[i]->sa_active_fd = fd;
 				return 1;
 			}
-			else
-				return 0;
 		}
 	}
 	/* did not match oid and passwd */
@@ -1471,7 +1475,7 @@ smux_peer_cleanup(int sd)
 		if (Auths[i]->sa_active_fd == sd) {
 		  	char oid_name[128];
 			Auths[i]->sa_active_fd = -1;
-			sprint_objid(oid_name, Auths[i]->sa_oid, Auths[i]->sa_oid_len);
+			snprint_objid(oid_name, sizeof(oid_name), Auths[i]->sa_oid, Auths[i]->sa_oid_len);
 			snmp_log(LOG_INFO, "peer disconnected: %s\n", oid_name);
 		}
 	}
