@@ -59,20 +59,20 @@
 #include <linux/if_tunnel.h>
 #include <linux/if_arp.h>
 
-#include "ucd-snmp/ucd-snmp-config.h"
-#include "ucd-snmp/snmp_alarm.h"
-#include "ucd-snmp/callback.h"
-#include "ucd-snmp/default_store.h"
-#include "ucd-snmp/asn1.h"
-#include "ucd-snmp/snmp.h"
-#include "ucd-snmp/snmp_debug.h"
-#include "ucd-snmp/snmp_api.h"
-#include "ucd-snmp/snmp_impl.h"
-#include "ucd-snmp/snmp_vars.h"
-#include "ucd-snmp/var_struct.h"
-#include "ucd-snmp/agent_read_config.h"
-#include "ucd-snmp/tools.h"
-#include "ucd-snmp/util_funcs.h"
+#include "config.h"
+#include "snmp_alarm.h"
+#include "callback.h"
+#include "default_store.h"
+#include "asn1.h"
+#include "snmp.h"
+#include "snmp_debug.h"
+#include "snmp_api.h"
+#include "snmp_impl.h"
+#include "snmp_vars.h"
+#include "var_struct.h"
+#include "agent_read_config.h"
+#include "tools.h"
+#include "util_funcs.h"
 
 #include "tunnel.h"
 
