@@ -34,8 +34,6 @@
 #include <dlfcn.h>
 #include "dlmod.h"
 
-static long long_return;
-
 static struct dlmod *dlmods = NULL;
 static int          dlmod_next_index = 1;
 static char         dlmod_path[1024];
@@ -102,7 +100,7 @@ dlmod_create_module (void)
     struct dlmod **pdlmod, *dlm;
 
     DEBUGMSGTL(("dlmod", "dlmod_create_module\n"));
-    dlm = calloc(1, sizeof(struct dlmod));
+    dlm = (struct dlmod *)calloc(1, sizeof(struct dlmod));
     if (dlm == NULL) 
 	return NULL;
 
@@ -368,9 +366,9 @@ var_dlmod(struct variable *vp,
 static struct dlmod *
 header_dlmodEntry(struct variable *vp,
 		  oid *name,
-		  int *length,
+		  size_t *length,
 		  int exact,
-		  int *var_len,
+		  size_t *var_len,
 		  WriteMethod **write_method)
 {
 #define DLMODENTRY_NAME_LENGTH 12
@@ -436,14 +434,14 @@ var_dlmodEntry(struct variable *vp,
     case DLMODNAME:
 	*write_method = write_dlmodName;
 	*var_len = strlen(dlm->name);
-	return dlm->name;
+	return (unsigned char*) dlm->name;
     case DLMODPATH:
 	*write_method = write_dlmodPath;
 	*var_len = strlen(dlm->path);
-	return dlm->path;
+	return (unsigned char*) dlm->path;
     case DLMODERROR:
 	*var_len = strlen(dlm->error);
-	return dlm->error;
+	return (unsigned char*) dlm->error;
     case DLMODSTATUS:
 	*write_method = write_dlmodStatus;
 	long_return = dlm->status;
@@ -478,7 +476,7 @@ write_dlmodName(int action,
 	dlm = dlmod_get_by_index(name[12]);
 	if (!dlm || dlm->status == DLMOD_LOADED) 
 	    return SNMP_ERR_RESOURCEUNAVAILABLE;
-	strncpy(dlm->name, var_val, var_val_len);
+	strncpy(dlm->name, (const char*)var_val, var_val_len);
 	dlm->name[var_val_len] = 0;
     }
     return SNMP_ERR_NOERROR;
@@ -507,7 +505,7 @@ write_dlmodPath(int action,
 	dlm = dlmod_get_by_index(name[12]);
 	if (!dlm || dlm->status == DLMOD_LOADED) 
 	    return SNMP_ERR_RESOURCEUNAVAILABLE;
-	strncpy(dlm->path, var_val, var_val_len);
+	strncpy(dlm->path, (const char*)var_val, var_val_len);
 	dlm->path[var_val_len] = 0;
     }
     return SNMP_ERR_NOERROR;
