@@ -150,7 +150,7 @@ void init_extensible(void)
   oid extensible_variables_oid[] = { EXTENSIBLEMIB,SHELLMIBNUM,1 };
 
   /* register ourselves with the agent to handle our mib tree */
-  REGISTER_MIB("ucd-snmp/extensible", extensible_extensible_variables, variable2, \
+  REGISTER_MIB("ucd-snmp/extensible", extensible_extensible_variables, variable2,
                extensible_variables_oid);
 
   snmpd_register_config_handler("exec", extensible_parse_config,
@@ -340,6 +340,8 @@ u_char *var_extensible_shell(struct variable *vp,
         else
           shell_command(exten);
         *var_len = strlen(exten->output);
+        if (exten->output[*var_len-1] == '\n')
+            exten->output[--(*var_len)] = '\0';
         return((u_char *) (exten->output));
       case ERRORFIX:
         *write_method = fixExecError;
@@ -381,7 +383,7 @@ fixExecError(int action,
       sprintf(ex.command, exten->fixcmd);
       if ((fd = get_exec_output(&ex))) {
         file = fdopen(fd,"r");
-        while (fgets(ex.output,STRMAX,file) != NULL);
+        while (fgets(ex.output,sizeof(ex.output),file) != NULL);
         fclose(file);
         wait_on_exec(&ex);
       }
@@ -489,6 +491,8 @@ u_char *var_extensible_relocatable(struct variable *vp,
         strcpy(errmsg,exten->output);
       }
       *var_len = strlen(errmsg);
+      if (errmsg[*var_len-1] == '\n')
+          errmsg[--(*var_len)] = '\0';
       return((u_char *) (errmsg));
     case ERRORFIX:
       *write_method = fixExecError;
