@@ -10,14 +10,12 @@ void init_proc(void);
 
 extern FindVarMethod var_extensible_proc;
 extern WriteMethod fixProcError;
-struct myproc *get_proc_instance (struct myproc*, int);
 int sh_count_procs (char *);
-int get_ps_output (struct extensible *);
 
 /* config file parsing routines */
 void proc_free_config (void);
-void proc_parse_config (char *, char *);
-void procfix_parse_config (char *, char *);
+void proc_parse_config (const char *, char *);
+void procfix_parse_config (const char *, char *);
 
 #include "mibdefs.h"
 
