@@ -45,6 +45,8 @@
 #define CPU_PRC         100
 #define BOOTTIME_SYMBOL	"boottime"
 
+static FindVarMethod var_extensible_vmstat;
+
 void init_vmstat_bsdi4(void)
 {
 
@@ -99,6 +101,7 @@ getuptime(void )
 	return(uptime);
 }
 
+static
 unsigned char *var_extensible_vmstat(struct variable *vp,
 				     oid *name,
 				     size_t *length,
