@@ -116,7 +116,7 @@ static void dump_var (
     temp_var.type = statType;
     temp_var.val.string = (u_char *)statP;
     temp_var.val_len = statLen;
-    sprint_variable (buf, var_name, var_name_len, &temp_var);
+    snprint_variable (buf, sizeof(buf), var_name, var_name_len, &temp_var);
     snmp_log(LOG_INFO, "    >> %s\n", buf);
 }
 
@@ -358,7 +358,7 @@ handle_snmp_packet(int operation, struct snmp_session *session, int reqid,
                    struct snmp_pdu *pdu, void *magic)
 {
     struct agent_snmp_session  *asp;
-    int status, allDone, i;
+    int status, allDone, i, error_index = 0;
     struct variable_list *var_ptr, *var_ptr2;
 
     if ( magic == NULL ) {
@@ -526,8 +526,10 @@ handle_snmp_packet(int operation, struct snmp_session *session, int reqid,
 
 	    status = handle_next_pass( asp );
 
-	    if ( status != SNMP_ERR_NOERROR )
+            if ( status != SNMP_ERR_NOERROR ){
 	        asp->mode = FREE;
+                error_index = asp->index;
+            }
 	    else
 	        asp->mode = RESERVE2;
 
@@ -538,8 +540,10 @@ handle_snmp_packet(int operation, struct snmp_session *session, int reqid,
 	if ( asp->mode == RESERVE2 ) {
 	    status = handle_next_pass( asp );
 
-	    if ( status != SNMP_ERR_NOERROR )
+	    if ( status != SNMP_ERR_NOERROR ){
 	        asp->mode = FREE;
+                error_index = asp->index;
+            }
 	    else
 	        asp->mode = ACTION;
 
@@ -550,8 +554,10 @@ handle_snmp_packet(int operation, struct snmp_session *session, int reqid,
 	if ( asp->mode == ACTION ) {
 	    status = handle_next_pass( asp );
 
-	    if ( status != SNMP_ERR_NOERROR )
+	    if ( status != SNMP_ERR_NOERROR ){
 	        asp->mode = UNDO;
+                error_index = asp->index;
+            }
 	    else
 	        asp->mode = COMMIT;
 
@@ -565,6 +571,7 @@ handle_snmp_packet(int operation, struct snmp_session *session, int reqid,
 	    if ( status != SNMP_ERR_NOERROR ) {
 		status    = SNMP_ERR_COMMITFAILED;
 	        asp->mode = FINISHED_FAILURE;
+                error_index = asp->index;
 	    }
 	    else
 	        asp->mode = FINISHED_SUCCESS;
@@ -574,18 +581,18 @@ handle_snmp_packet(int operation, struct snmp_session *session, int reqid,
 	}
 
 	if ( asp->mode == UNDO ) {
-	    if (handle_next_pass( asp ) != SNMP_ERR_NOERROR )
+            if (handle_next_pass( asp ) != SNMP_ERR_NOERROR ) {
 		status = SNMP_ERR_UNDOFAILED;
+                error_index = 0;
+            }
 
 	    asp->mode = FINISHED_FAILURE;
-	    break;
 	}
 
 	if ( asp->mode == FREE ) {
 	    (void) handle_next_pass( asp );
-	    break;
 	}
-
+        asp->index = error_index;
 	break;
 
     case SNMP_MSG_RESPONSE:
@@ -660,6 +667,48 @@ handle_snmp_packet(int operation, struct snmp_session *session, int reqid,
 		    }
 	    }
 	}
+
+	    /*
+             * Update the snmp error-count statistics
+             *   XXX - should we include the V2 errors in this or not?
+             */
+#define INCLUDE_V2ERRORS_IN_V1STATS
+
+	switch ( status ) {
+#ifdef INCLUDE_V2ERRORS_IN_V1STATS
+	    case SNMP_ERR_WRONGVALUE:
+	    case SNMP_ERR_WRONGENCODING:
+	    case SNMP_ERR_WRONGTYPE:
+	    case SNMP_ERR_WRONGLENGTH:
+	    case SNMP_ERR_INCONSISTENTVALUE:
+#endif
+	    case SNMP_ERR_BADVALUE:
+                snmp_increment_statistic(STAT_SNMPOUTBADVALUES);
+	        break;
+#ifdef INCLUDE_V2ERRORS_IN_V1STATS
+	    case SNMP_ERR_NOACCESS:
+	    case SNMP_ERR_NOTWRITABLE:
+	    case SNMP_ERR_NOCREATION:
+	    case SNMP_ERR_INCONSISTENTNAME:
+	    case SNMP_ERR_AUTHORIZATIONERROR:
+#endif
+	    case SNMP_ERR_NOSUCHNAME:
+                snmp_increment_statistic(STAT_SNMPOUTNOSUCHNAMES);
+	        break;
+#ifdef INCLUDE_V2ERRORS_IN_V1STATS
+	    case SNMP_ERR_RESOURCEUNAVAILABLE:
+	    case SNMP_ERR_COMMITFAILED:
+	    case SNMP_ERR_UNDOFAILED:
+#endif
+	    case SNMP_ERR_GENERR:
+                snmp_increment_statistic(STAT_SNMPOUTGENERRS);
+	        break;
+
+	    case SNMP_ERR_TOOBIG:
+                snmp_increment_statistic(STAT_SNMPOUTTOOBIGS);
+	        break;
+	}
+
 	if (( status == SNMP_ERR_NOERROR ) && ( asp->pdu )) {
 	    snmp_increment_statistic_by(
 		(asp->pdu->command == SNMP_MSG_SET ?
@@ -678,7 +727,11 @@ handle_snmp_packet(int operation, struct snmp_session *session, int reqid,
 	if ( asp->pdu ) {
 	    asp->pdu->command  = SNMP_MSG_RESPONSE;
 	    asp->pdu->errstat  = status;
-	    asp->pdu->errindex = asp->index;
+	    if (status == SNMP_ERR_NOERROR) {
+		asp->pdu->errindex = 0;
+	    } else {
+		asp->pdu->errindex = asp->index;
+	    }
 	    if (! snmp_send( asp->session, asp->pdu ))
 	        snmp_free_pdu(asp->pdu);
 	    snmp_increment_statistic(STAT_SNMPOUTPKTS);
@@ -755,6 +808,8 @@ handle_var_list(struct agent_snmp_session  *asp)
 {
     struct variable_list *varbind_ptr;
     int     status;
+    int     saved_status = SNMP_ERR_NOERROR;
+    int     saved_index;
     int     count;
 
     count = 0;
@@ -769,7 +824,12 @@ handle_var_list(struct agent_snmp_session  *asp)
 	status = handle_one_var(asp, varbind_ptr);
 
 	if ( status != SNMP_ERR_NOERROR ) {
-	    return status;
+	    if (asp->rw == WRITE ) {
+	        saved_status = status;
+	        saved_index  = count;
+	    }
+	    else
+	        return status;
 	}
 
 	if ( varbind_ptr == asp->end )
@@ -777,9 +837,11 @@ handle_var_list(struct agent_snmp_session  *asp)
 	varbind_ptr = varbind_ptr->next_variable;
 	if ( asp->mode == RESERVE1 )
 	    snmp_vars_inc++;
-   }
-   asp->index = 0;
-   return SNMP_ERR_NOERROR;
+    }
+    if (saved_status != SNMP_ERR_NOERROR ) {
+       asp->index = saved_index;
+    }
+    return saved_status;
 }
 
 int
