@@ -349,7 +349,7 @@ search_subtree_vars(struct subtree *tp,
 			 */
 		    if ( access && snmp_oid_compare(name, *namelen,
 						    tp->end, tp->end_len) > 0) {
-			memcpy(name, tp->end, tp->end_len);
+			memcpy(name, tp->end, tp->end_len * sizeof(oid));
 			access = 0;
 		    }
 #if MIB_CLIENTS_ARE_EVIL
@@ -370,7 +370,8 @@ search_subtree_vars(struct subtree *tp,
                     /* check for permission to view this part of the OID tree */
 		    if ((access != NULL || (*write_method != NULL && exact)) &&
                         in_a_view(name, namelen, pdu, cvp->type)) {
-			if ( access && !exact ) {
+			if ( access && !exact &&
+                             (access!=agentx_add_request) ) {
 				/*
 				 * We've got an answer, but shouldn't use it.
 				 * But we *might* be able to use a later
@@ -406,6 +407,7 @@ search_subtree_vars(struct subtree *tp,
 	    if (access != NULL || (exact && *write_method != NULL)) {
 	        *type = cvp->type;
 		*acl = cvp->acl;
+		*noSuchObject = FALSE;
 		return access;
 	    }
 	    return NULL;
