@@ -161,6 +161,15 @@ typedef long    fd_mask;
 
 #include "mib_module_includes.h"
 
+/*
+ * Include winservice.h to support Windows Service
+ */
+#ifdef WIN32
+#include <windows.h>
+#include <tchar.h>
+#include "winservice.h"
+#endif
+
 /*
  * Globals.
  */
@@ -179,6 +188,13 @@ int 		snmp_dump_packet;
 int             running          = 1;
 int		reconfig	 = 0;
 
+#ifdef WIN32
+/* SNMP Agent Status */
+#define AGENT_RUNNING 1
+#define AGENT_STOPPED 0
+int agent_status = AGENT_STOPPED;
+#endif
+
 struct addrCache {
     in_addr_t	addr;
     int		status;
@@ -203,28 +219,70 @@ static int sdlist[NUM_SOCKETS], sdlen = 0;
 #endif /* USING_SMUX_MODULE */
 
 /*
- * Prototypes.
+ * Declare Windows Service related global variables
+ */
+#ifdef WIN32
+LPTSTR g_szAppName = _T("Net-Snmp Agent");   /* Application Name */
+#endif
+
+/*
+ * Prototypes
  */
 int snmp_read_packet (int);
 int snmp_input (int, struct snmp_session *, int, struct snmp_pdu *, void *);
 static void usage (char *);
+
+#ifdef WIN32
+int __cdecl _tmain(int argc, TCHAR *argv[]);
+#else
 int main (int, char **);
+#endif
+
 static void SnmpTrapNodeDown (void);
 static int receive(void);
 int snmp_check_packet(struct snmp_session*, snmp_ipaddr);
 int snmp_check_parse(struct snmp_session*, struct snmp_pdu*, int);
 
+#ifdef WIN32
+/* Stop Function to break the infinite loop
+ * This is requried to stop proccess, when STOP request
+ * received from the SCM
+ */
+void StopSnmpAgent(void);
+/*
+ * Main Snmp Deamon
+ * Moving all main() code to this function to support
+ * Windows Serivce functionality
+ */
+int SnmpDaemonMain(int argc, TCHAR *argv[]);
+#endif
+
+
 static void usage(char *prog)
 {
+#ifdef WIN32
+printf("\nUsage:  %s -register [param list] |",prog);
+printf("\n\t\t-unregister |"); 
+printf("\n\t\t[-h] [-v] [-f] [-a] [-d] [-V] [-P PIDFILE] [-q] [-D] [-p NUM] [-L] [-l LOGFILE] [-r]");
+#else
 	printf("\nUsage:  %s [-h] [-v] [-f] [-a] [-d] [-V] [-P PIDFILE] [-q] [-D] [-p NUM] [-L] [-l LOGFILE] [-r]",prog);
+#endif /* WIN32 */
 #if HAVE_UNISTD_H
 	printf(" [-u uid] [-g gid]");
 #endif
 	printf("\n");
 	printf("\n\tVersion:  %s\n",VersionInfo);
-	printf("\tAuthor:   Wes Hardaker\n");
 	printf("\tEmail:    net-snmp-coders@lists.sourceforge.net\n");
 	printf("\n-h\t\tThis usage message.\n");
+#ifdef WIN32
+	printf("-register [param list]");
+	printf("\n\t\tRegister as windows service");
+	printf("\n\t\t\"param list\"\tStartup parameter list for service, same as normal parameters");
+	printf("\n\t\tE.g.: %s -register -p 2002",prog);
+	printf("\n\t\tThis registers %s as service, which listens on port 2002",prog);
+	printf("\n\t\tNote:- Some options doesn't make sense when running as service");
+	printf("\n-unregister\tUnregisters service, if already registered\n");
+#endif
 	printf("-H\t\tDisplay configuration file directives understood.\n");
 	printf("-v\t\tVersion information.\n");
 	printf("-f\t\tDon't fork from the shell.\n");
@@ -248,7 +306,7 @@ static void usage(char *prog)
 	printf("-A\t\tAppend to the logfile rather than truncating it.\n");
 	printf("-r\t\tDon't exit if root only accessible files can't be opened\n");
 	printf("-I [-]INITLIST\tList of mib modules to initialize (or not).\n");
-	printf("\t\t (run snmpd with -Dinit_mib for a list)\n");
+	printf("\t\t (run snmpd with -Dmib_init for a list)\n");
 	printf("-l LOGFILE\tPrint warnings/messages to LOGFILE\n");
 #ifdef LOGFILE
 	printf("\t\t(By default LOGFILE=%s)\n", LOGFILE);
@@ -267,7 +325,16 @@ static void usage(char *prog)
 	RETSIGTYPE
 SnmpdShutDown(int a)
 {
+        extern struct snmp_session *main_session;
 	running = 0;
+#ifdef WIN32
+	/*
+	 * In case of windows, select() in receive() function will not return 
+	 * on signal. Thats why following function is called, which closes the 
+	 * main socket descriptor and causes the select() to return
+	*/
+	snmp_close(main_session);
+#endif
 }
 
 #ifdef SIGHUP
@@ -298,7 +365,8 @@ SnmpTrapNodeDown(void)
 }
 
 /*******************************************************************-o-******
- * main
+ * main - Non Windows
+ * SnmpDeamonMain - Windows to support windows serivce
  *
  * Parameters:
  *	 argc
@@ -312,8 +380,12 @@ SnmpTrapNodeDown(void)
  *
  * Also successfully EXITs with zero for some options.
  */
-	int
+int
+#ifdef WIN32
+SnmpDaemonMain(int argc, TCHAR *argv[])
+#else
 main(int argc, char *argv[])
+#endif
 {
 	int             arg, i;
 	int             ret;
@@ -328,7 +400,7 @@ main(int argc, char *argv[])
 	int             dont_zero_log = 0;
 	int             stderr_log=0, syslog_log=0;
 	int             uid=0, gid=0;
-        int             agent_mode=-1;
+        int             agent_mode=-1;	
 
 #ifdef LOGFILE
 	strncpy(logfile, LOGFILE, PATH_MAX);
@@ -341,6 +413,7 @@ main(int argc, char *argv[])
 			/* Default to NOT running an AgentX master */
         ds_set_boolean(DS_APPLICATION_ID, DS_AGENT_AGENTX_MASTER, 0);
 
+
 	/*
 	 * usage: snmpd
 	 */
@@ -529,15 +602,13 @@ main(int argc, char *argv[])
                   exit(0);
                 case 'v':
                   printf("\nUCD-snmp version:  %s\n",VersionInfo);
-                  printf("Author:            Wes Hardaker\n");
-                  printf("Email:             ucd-snmp-coders@ucd-snmp.ucdavis.edu\n\n");
+                  printf("Email:             net-snmp-coders@lists.sourceforge.net\n\n");
                   exit (0);
                 case '-':
                   switch(argv[arg][2]){
                     case 'v': 
                       printf("\nUCD-snmp version:  %s\n",VersionInfo);
-                      printf("Author:            Wes Hardaker\n");
-                      printf("Email:             ucd-snmp-coders@ucd-snmp.ucdavis.edu\n\n");
+                      printf("Email:             net-snmp-coders@lists.sourceforge.net\n\n");
                       exit (0);
                     case 'h':
                       usage(argv[0]);
@@ -607,25 +678,15 @@ main(int argc, char *argv[])
     }
 #endif
 
-#if HAVE_GETPID
-    if (pid_file != NULL) {
-      if ((PID = fopen(pid_file, "w")) == NULL) {
-        snmp_log_perror("fopen");
-        if (!ds_get_boolean(DS_APPLICATION_ID, DS_AGENT_NO_ROOT_ACCESS))
-          exit(1);
-      }
-      else {
-        fprintf(PID, "%d\n", (int)getpid());
-        fclose(PID);
-      }
-    }
-#endif
-
     /*  Honor selection of standard error output.  */
     if (!stderr_log) {
       snmp_disable_stderrlog();
     }
 
+#if defined(SIGPIPE) && defined(SIG_IGN)
+    signal(SIGPIPE, SIG_IGN);  /* 'Inline' failure of wayward readers */
+#endif
+
     SOCK_STARTUP;
     init_agent("snmpd");		/* do what we need to do first. */
     init_mib_modules();
@@ -652,9 +713,6 @@ main(int argc, char *argv[])
 #ifdef SIGUSR1
     signal(SIGUSR1, SnmpdDump);
 #endif
-#ifdef SIGPIPE
-    signal(SIGPIPE, SIG_IGN);	/* 'Inline' failure of wayward readers */
-#endif
 
     /* store persistent data immediately in case we crash later */
     snmp_store("snmpd");
@@ -662,6 +720,20 @@ main(int argc, char *argv[])
     /* send coldstart trap via snmptrap(1) if possible */
     send_easy_trap (0, 0);
         
+#if HAVE_GETPID
+    if (pid_file != NULL) {
+      if ((PID = fopen(pid_file, "w")) == NULL) {
+        snmp_log_perror("fopen");
+        if (!ds_get_boolean(DS_APPLICATION_ID, DS_AGENT_NO_ROOT_ACCESS))
+          exit(1);
+      }
+      else {
+        fprintf(PID, "%d\n", (int)getpid());
+        fclose(PID);
+      }
+    }
+#endif
+
 #if HAVE_UNISTD_H
 #ifdef HAVE_SETGID
 	if ((gid = ds_get_int(DS_APPLICATION_ID, DS_AGENT_GROUPID)) != 0) {
@@ -691,7 +763,10 @@ main(int argc, char *argv[])
 
 	/* we're up, log our version number */
 	snmp_log(LOG_INFO, "UCD-SNMP version %s\n", VersionInfo);
-
+#ifdef WIN32
+	/* SNMP Agent started, set the status to running */
+	agent_status = AGENT_RUNNING;
+#endif	
 	memset(addrCache, 0, sizeof(addrCache));
 	/* 
 	 * Forever monitor the dest_port for incoming PDUs.
@@ -703,6 +778,9 @@ main(int argc, char *argv[])
 	SnmpTrapNodeDown();
 	DEBUGMSGTL(("snmpd", "Bye...\n"));
 	snmp_shutdown("snmpd");
+#ifdef WIN32
+	agent_status = AGENT_STOPPED; 
+#endif
 	return 0;
 
 }  /* End main() -- snmpd */
@@ -806,6 +884,16 @@ receive(void)
 	
 	count = select(numfds, &readfds, &writefds, &exceptfds, tvp);
 
+#ifdef WIN32
+        /*
+         * Check if select() returned on signal. 
+         * In case of windows to return from select() on signal, closesocket 
+         * is used which causes the select() to return with count>0.  
+        */
+	if(!running)
+	   break;
+#endif
+
 	if (count > 0) {
 
 #ifdef USING_SMUX_MODULE 
@@ -1027,7 +1115,8 @@ snmp_check_parse( struct snmp_session *session,
 	    for ( var_ptr = pdu->variables ;
 	        var_ptr != NULL ; var_ptr=var_ptr->next_variable )
 	    {
-                sprint_objid (c_oid, var_ptr->name, var_ptr->name_length);
+                snprint_objid (c_oid, sizeof(c_oid),
+                               var_ptr->name, var_ptr->name_length);
                 snmp_log(LOG_DEBUG, "    -- %s\n", c_oid);
 	    }
 	}
@@ -1079,3 +1168,79 @@ snmp_input(int op,
     return 1;
 
 } /* end snmp_input() */
+
+
+
+/* Windows Service Related functions */
+#ifdef WIN32
+/************************************************************
+* main function for Windows
+* Parse command line arguments for startup options,
+* to start as service or console mode application in windows.
+* Invokes appropriate startup funcitons depending on the 
+* parameters passesd
+*************************************************************/
+int
+__cdecl _tmain(int argc, TCHAR *argv[])
+{
+
+	/* Define Service Name and Description, which appears in windows SCM */
+	LPCTSTR lpszServiceName = g_szAppName; /* Service Registry Name */
+	LPCTSTR lpszServiceDisplayName = _T("Net SNMP Agent Daemon"); /* Display Name */
+	LPCTSTR lpszServiceDescription = _T("SNMP agent for windows from Net-SNMP");
+	InputParams InputOptions;
+
+
+	int nRunType = RUN_AS_CONSOLE;
+	nRunType = ParseCmdLineForServiceOption(argc,argv);
+
+	switch(nRunType)
+	{
+		case REGISTER_SERVICE:
+			/* Register As service */
+			InputOptions.Argc = argc;
+			InputOptions.Argv = argv;
+			RegisterService(lpszServiceName,
+							lpszServiceDisplayName,
+							lpszServiceDescription,
+							&InputOptions);
+			exit(0);
+			break;
+		case UN_REGISTER_SERVICE:
+			/* Unregister service */
+			UnregisterService(lpszServiceName);
+			exit(0);
+			break;
+		case RUN_AS_SERVICE:
+			/* Run as service */
+			/* Register Stop Function */
+			RegisterStopFunction(StopSnmpAgent);
+			return RunAsService(SnmpDaemonMain);
+			break;
+		default:
+			/* Run Net-Snmpd in console mode */
+			/* Invoke SnmpDeamonMain with input arguments */
+			return SnmpDaemonMain(argc,argv);
+			break;
+	}
+}
+
+/*
+ * To stop Snmp Agent deamon 
+ * This portion is still not working
+ */
+void StopSnmpAgent(void)
+{
+	/* Shut Down Agent */
+	SnmpdShutDown(1);
+
+	/* Wait till agent is completely stopped */
+
+	while(agent_status != AGENT_STOPPED)
+	{
+		Sleep(100);
+	}
+}
+
+#endif   /* if WIN32 */
+
