@@ -238,9 +238,11 @@ void print_log(char *file, char *message)
 void sprint_descriptor(char *buffer,
 		       struct varInfo *vip)
 {
-  char buf[SPRINT_MAX_LEN], *cp;
+  char *buf = malloc(SPRINT_MAX_LEN), *cp;
+  size_t buf_len = SPRINT_MAX_LEN, str_len = 0;
 
-  sprint_objid(buf, vip->info_oid, vip->oidlen);
+  sprint_realloc_objid((u_char **)&buf, &buf_len, &str_len, 1, 
+	  vip->info_oid, vip->oidlen);
   for(cp = buf; *cp; cp++)
     ;
   while(cp >= buf){
@@ -257,12 +259,13 @@ void sprint_descriptor(char *buffer,
   if (cp < buf)
     cp = buf;
   strcpy(buffer, cp);
+  free(buf);
 }
 
 void processFileArgs(char *fileName)
 {
   FILE *fp;
-  char buf[260], *cp;
+  char buf[260] = { 0 }, *cp;
   int blank, linenumber = 0;
 
   fp = fopen(fileName, "r");
@@ -362,11 +365,11 @@ int main(int argc, char *argv[])
   time_t this_time;
   time_t delta_time;
   int sum; /* what the heck is this for, its never used? */
-  char filename[128];
+  char filename[128] = { 0 };
   struct timeval tv;
   struct tm tm;
-  char timestring[64], valueStr[64], maxStr[64], peakStr[64];
-  char outstr[256];
+  char timestring[64] = { 0 }, valueStr[64] = { 0 }, maxStr[64] = { 0 };
+  char outstr[256] = { 0 }, peakStr[64] = { 0 };
   int status;
   int begin, end, last_end;
   int print = 1;
@@ -484,9 +487,13 @@ int main(int argc, char *argv[])
 	  if (!vars){
 	    fprintf(stderr, "Missing variable in reply\n");
 	    continue;
-	  } else {
-	    this_time = *(vars->val.integer);
 	  }
+	  else if (vars->type == SNMP_NOSUCHOBJECT ||
+		   vars->type == SNMP_NOSUCHINSTANCE) {
+	    fprint_variable(stderr, vars->name, vars->name_length, vars);
+	    continue;
+	  }
+	  this_time = *(vars->val.integer);
 	  vars = vars->next_variable;
 	} else {
 	  this_time = 1;
@@ -500,6 +507,11 @@ int main(int argc, char *argv[])
 	      fprintf(stderr, "Missing variable in reply\n");
 	      break;
 	    }
+	    else if (vars->type == SNMP_NOSUCHOBJECT ||
+		     vars->type == SNMP_NOSUCHINSTANCE) {
+	      fprint_variable(stderr, vars->name, vars->name_length, vars);
+	      break;
+	    }
 	    vip->type = vars->type;
             if (vars->type == ASN_COUNTER64) {
               u64Subtract(vars->val.counter64, &vip->c64value, &c64value);
@@ -527,11 +539,12 @@ int main(int argc, char *argv[])
           }
 
 	  if (tableForm) {
-	    if (count == begin) sprintf(outstr, "%s", timestring+1);
-	    else outstr[0] = 0;
-	  }
-	  else
+	    if (count == begin) {
+	      sprintf(outstr, "%s", timestring+1);
+	    }
+	  } else {
 	    sprintf(outstr, "%s %s", timestring, vip->descriptor);
+	  }
 
 	  if (deltat || tableForm){
             if (vip->type == ASN_COUNTER64) {
@@ -632,7 +645,7 @@ int main(int argc, char *argv[])
 	    ;
 	  if (vars)
 	    fprint_objid(stderr, vars->name, vars->name_length);
-	  fprintf(stderr, "\n");
+	  else fprintf(stderr, "\n");
 /* Don't exit when OIDs from file are not found on agent
 	  exit_code = 1;
 	  break;
