@@ -56,6 +56,12 @@ SOFTWARE.
 #include <in.h>
 #endif
 
+#ifdef dynix
+#if HAVE_SYS_SELECT_H
+#include <sys/select.h>
+#endif
+#endif
+
 #if HAVE_DMALLOC_H
 #include <dmalloc.h>
 #endif
@@ -510,7 +516,7 @@ asn_parse_string(u_char *data,
 
     DEBUGIF("dumpv_recv") {
       char *buf = (char *)malloc(1+asn_length);
-      sprint_asciistring(buf, string, asn_length);
+      snprint_asciistring(buf, asn_length+1, string, asn_length);
       DEBUGMSG(("dumpv_recv", "  String:\t%s\n", buf));
       free (buf);
     }
@@ -567,7 +573,7 @@ asn_build_string(u_char *data,
     DEBUGDUMPSETUP("send", initdatap, data - initdatap + strlength);
     DEBUGIF("dumpv_send") {
       char *buf = (char *)malloc(1+strlength);
-      sprint_asciistring(buf, string, strlength);
+      snprint_asciistring(buf, strlength+1, string, strlength);
       DEBUGMSG(("dumpv_send", "  String:\t%s\n", buf));
       free (buf);
     }
@@ -1031,6 +1037,10 @@ asn_build_objid(u_char *data,
     }
     first_objid_val = objid_val;
 
+    /* ditch illegal calls now */
+    if (objidlength > MAX_OID_LEN)
+        return NULL;
+        
     /* calculate the number of bytes needed to store the encoded value */
     for (i = 1, asnlength = 0;;) {
         if (objid_val < (unsigned)0x80) {
@@ -1272,7 +1282,9 @@ asn_build_bitstring(u_char *data,
  * ASN.1 bit string ::= 0x03 asnlength unused {byte}*
  */
     static const char *errpre = "build bitstring";
-    if (_asn_bitstring_check(errpre, strlength, ((string) ? *string : 0)))
+	u_char ufc = (u_char)0;
+	if (string) ufc = *string;
+    if (_asn_bitstring_check(errpre, strlength, ufc))
 	return NULL;
 
     data = asn_build_header(data, datalength, type, strlength);
@@ -1321,7 +1333,7 @@ asn_rbuild_bitstring(u_char *data,
     DEBUGDUMPSETUP("send", data+1, initdatap - data);
     DEBUGIF("dumpv_send") {
       char *buf = (char *)malloc(2*strlength);
-      sprint_asciistring(buf, string, strlength);
+      snprint_asciistring(buf, 2*strlength, string, strlength);
       DEBUGMSG(("dumpv_send", "  Bitstring:\t%s\n", buf));
       free (buf);
     }
@@ -2149,7 +2161,7 @@ asn_rbuild_string (u_char *data,
             DEBUGMSG(("dumpv_send", "  String: [NULL]\n"));
         } else {
             char *buf = (char *)malloc(2*strlength);
-            sprint_asciistring(buf, string, strlength);
+            snprint_asciistring(buf, 2*strlength, string, strlength);
             DEBUGMSG(("dumpv_send", "  String:\t%s\n", buf));
             free (buf);
         }
@@ -2208,7 +2220,7 @@ asn_rbuild_header (u_char *data,
     char ebuf[128];
     
     data = asn_rbuild_length(data, datalength, length);
-    if (*datalength < 1){
+    if (*datalength < 1 || data == NULL){
 	sprintf(ebuf, "bad header length < 1 :%d, %d", *datalength, length);
 	ERROR_MSG(ebuf);
 	return NULL;
