@@ -67,6 +67,7 @@ struct config_line *register_app_premib_handler (const char *,
                                              const char *);
 void unregister_config_handler (const char *, const char *);
 void unregister_app_config_handler (const char *);
+void unregister_all_config_handlers(void);
 void read_config_print_usage(const char *lead);
 char *read_config_save_octet_string(char *saveto, u_char *str, size_t len);
 char *read_config_read_octet_string(char *readfrom, u_char **str, size_t *len);
@@ -78,6 +79,12 @@ void  read_config_store(const char *type, const char *line);
 void  read_app_config_store(const char *line);
 void  snmp_save_persistent(const char *type);
 void  snmp_clean_persistent(const char *type);
+struct config_line *read_config_get_handlers(const char *type);
+
+void set_configuration_directory(const char *dir);
+const char *get_configuration_directory(void);
+void set_persistent_directory(const char *dir);
+const char *get_persistent_directory(void);
 
 #ifdef __cplusplus
 }
