@@ -104,7 +104,7 @@ snmpv3_authtype_conf(const char *word, char *cptr)
   if (strcasecmp(cptr,"MD5") == 0)
     defaultAuthType = usmHMACMD5AuthProtocol;
   else if (strcasecmp(cptr,"SHA") == 0)
-    defaultAuthType = usmHMACMD5AuthProtocol;
+    defaultAuthType = usmHMACSHA1AuthProtocol;
   else
     config_perror("Unknown authentication type");
   defaultAuthTypeLen = USM_LENGTH_OID_TRANSFORM;
@@ -137,8 +137,8 @@ snmpv3_privtype_conf(const char *word, char *cptr)
 oid *
 get_default_privtype(size_t *len)
 {
-  if (defaultAuthType == NULL) {
-    defaultAuthType = usmDESPrivProtocol;
+  if (defaultPrivType == NULL) {
+    defaultPrivType = usmDESPrivProtocol;
     defaultPrivTypeLen = USM_LENGTH_OID_TRANSFORM;
   }
   if (len)
@@ -705,8 +705,7 @@ init_snmpv3(const char *type) {
 
   gettimeofday(&snmpv3starttime, NULL);
 
-  if (!type)
-      type = ds_get_string(DS_LIBRARY_ID, DS_LIB_APPTYPE);
+  if (!type) type = "__snmpapp__";
 
   /* we need to be called back later */
   snmp_register_callback(SNMP_CALLBACK_LIBRARY, SNMP_CALLBACK_POST_READ_CONFIG,
@@ -715,9 +714,8 @@ init_snmpv3(const char *type) {
                          SNMP_CALLBACK_POST_PREMIB_READ_CONFIG,
                          init_snmpv3_post_premib_config, NULL);
   /* we need to be called back later */
-  if (type)
-      snmp_register_callback(SNMP_CALLBACK_LIBRARY, SNMP_CALLBACK_STORE_DATA,
-                             snmpv3_store, (void *) strdup(type));
+  snmp_register_callback(SNMP_CALLBACK_LIBRARY, SNMP_CALLBACK_STORE_DATA,
+                         snmpv3_store, (void *) strdup(type));
 
   /* initialize submodules */
   /*   NOTE: this must be after the callbacks are registered above,
@@ -732,16 +730,12 @@ init_snmpv3(const char *type) {
   /* register all our configuration handlers (ack, there's a lot) */
 
   /* handle engineID setup before everything else which may depend on it */
-  if (type) {
-      register_premib_handler(type,"engineID", engineID_conf, NULL, "string");
-      register_premib_handler(type,"oldEngineID", oldengineID_conf, NULL, NULL);
-      register_premib_handler(type,"engineIDType", engineIDType_conf, NULL,
-                              "num");
-      register_premib_handler(type,"engineIDNic", engineIDNic_conf, NULL,
-                              "string");
-      register_config_handler(type,"engineBoots", engineBoots_conf, NULL, NULL);
-  }
-  
+  register_premib_handler(type,"engineID", engineID_conf, NULL, "string");
+  register_premib_handler(type,"oldEngineID", oldengineID_conf, NULL, NULL);
+  register_premib_handler(type,"engineIDType", engineIDType_conf, NULL,"num");
+  register_premib_handler(type,"engineIDNic", engineIDNic_conf, NULL,"string");
+  register_config_handler(type,"engineBoots", engineBoots_conf, NULL, NULL);
+
   /* default store config entries */
   ds_register_config(ASN_OCTET_STR, "snmp", "defSecurityName", DS_LIBRARY_ID,
                      DS_LIB_SECNAME);
@@ -761,21 +755,18 @@ init_snmpv3(const char *type) {
                           "DES (currently the only possible value)");
   register_config_handler("snmp","defSecurityLevel", snmpv3_secLevel_conf,
                           NULL, "noAuthNoPriv|authNoPriv|authPriv");
-
-  if (type) {
-      register_config_handler(type,"userSetAuthPass", usm_set_password, NULL,
-                              NULL);
-      register_config_handler(type,"userSetPrivPass", usm_set_password, NULL,
-                              NULL);
-      register_config_handler(type,"userSetAuthKey", usm_set_password, NULL,
-                              NULL);
-      register_config_handler(type,"userSetPrivKey", usm_set_password, NULL,
-                              NULL);
-      register_config_handler(type,"userSetAuthLocalKey", usm_set_password,
-                              NULL, NULL);
-      register_config_handler(type,"userSetPrivLocalKey", usm_set_password,
-                              NULL, NULL);
-  }
+  register_config_handler(type,"userSetAuthPass", usm_set_password, NULL,
+                          NULL);
+  register_config_handler(type,"userSetPrivPass", usm_set_password, NULL,
+                          NULL);
+  register_config_handler(type,"userSetAuthKey", usm_set_password, NULL,
+                          NULL);
+  register_config_handler(type,"userSetPrivKey", usm_set_password, NULL,
+                          NULL);
+  register_config_handler(type,"userSetAuthLocalKey", usm_set_password, NULL,
+                          NULL);
+  register_config_handler(type,"userSetPrivLocalKey", usm_set_password, NULL,
+                          NULL);
 }
 
 /*
