@@ -256,7 +256,8 @@ vacm_parse_config_group(const char *token, char *line)
 struct vacm_viewEntry *
 vacm_getViewEntry(const char *viewName,
 		  oid *viewSubtree,
-		  size_t viewSubtreeLen)
+		  size_t viewSubtreeLen,
+		  int ignoreMask)
 {
     struct vacm_viewEntry *vp, *vpret = NULL;
     char view[VACMSTRINGLEN];
@@ -269,20 +270,24 @@ vacm_getViewEntry(const char *viewName,
     strcpy(view+1, viewName);
     for(vp = viewList; vp; vp = vp->next){
         if (!memcmp(view, vp->viewName,glen+1)
-	    && viewSubtreeLen >= vp->viewSubtreeLen) {
+	    && viewSubtreeLen >= (vp->viewSubtreeLen-1)) {
 	    int mask = 0x80, maskpos = 0;
 	    int oidpos;
             found = 1;
-	    for (oidpos = 0; found && oidpos < (int)vp->viewSubtreeLen; oidpos++) {
-		if ((vp->viewMask[maskpos] & mask) != 0) {
-		    if (viewSubtree[oidpos] != vp->viewSubtree[oidpos])
-                        found = 0;
+
+	    if (!ignoreMask) {
+		for (oidpos = 0; found && oidpos < (int)vp->viewSubtreeLen-1;
+		     oidpos++) {
+		    if ((vp->viewMask[maskpos] & mask) != 0) {
+			if (viewSubtree[oidpos] != vp->viewSubtree[oidpos+1])
+			    found = 0;
+		    }
+		    if (mask == 1) {
+			mask = 0x80;
+			maskpos++;
+		    }
+		    else mask >>= 1;
 		}
-		if (mask == 1) {
-		    mask = 0x80;
-		    maskpos++;
-		}
-		else mask >>= 1;
 	    }
             if (found) {
               /* match successful, keep this node if its longer than
@@ -290,9 +295,9 @@ vacm_getViewEntry(const char *viewName,
                  than the previous). */
               if (vpret == NULL || vp->viewSubtreeLen > vpret->viewSubtreeLen ||
                   (vp->viewSubtreeLen == vpret->viewSubtreeLen &&
-                   snmp_oid_compare(vp->viewSubtree, vp->viewSubtreeLen,
-                                    vpret->viewSubtree,
-                                    vpret->viewSubtreeLen) > 0))
+                   snmp_oid_compare(vp->viewSubtree+1, vp->viewSubtreeLen-1,
+                                    vpret->viewSubtree+1,
+                                    vpret->viewSubtreeLen-1) > 0))
                 vpret = vp;
             }
 	}
@@ -337,18 +342,17 @@ vacm_createViewEntry(const char *viewName,
 
     vp->viewName[0] = glen;
     strcpy(vp->viewName+1, viewName);
-    memcpy(vp->viewSubtree, viewSubtree, viewSubtreeLen * sizeof(oid));
-    vp->viewSubtreeLen = viewSubtreeLen;
+    vp->viewSubtree[0] = viewSubtreeLen;
+    memcpy(vp->viewSubtree+1, viewSubtree, viewSubtreeLen * sizeof(oid));
+    vp->viewSubtreeLen = viewSubtreeLen+1;
 
     lp = viewList;
     while (lp) {
 	cmp = memcmp(lp->viewName, vp->viewName, glen+1);
-	cmp2= memcmp(lp->viewSubtree,viewSubtree,sizeof(oid)*viewSubtreeLen);
-	if (cmp==0 && cmp2>0) break;
+	cmp2 = snmp_oid_compare(lp->viewSubtree, lp->viewSubtreeLen,
+	                        vp->viewSubtree, vp->viewSubtreeLen);
+	if (cmp == 0 && cmp2 > 0) break;
 	if (cmp > 0) break;
-	if (cmp < 0) goto next;
-	
-next:
 	op = lp;
 	lp = lp->next;
     }
@@ -665,3 +669,12 @@ store_vacm(int majorID, int minorID, void *serverarg, void *clientarg)
   return SNMPERR_SUCCESS;
 }
 
+/* returns 1 if vacm has *any* configuration entries in it (regardless
+   of weather or not there is enough to make a decision based on it),
+   else return 0 */
+int vacm_is_configured(void) {
+    if (viewList == NULL && accessList == NULL && groupList == NULL)
+        return 0;
+    return 1;
+}
+
